@@ -17,9 +17,11 @@ from devpi_common.metadata import BasenameMeta
 from devpi_common.metadata import is_archive_of_project
 from devpi_common.validation import normalize_name
 from functools import partial
+from .config import hookimpl
+from .model import BaseStageCustomizer
 from .model import BaseStage, make_key_and_href, SimplelinkMeta
+from .model import ensure_boolean
 from .model import join_requires
-from .model import InvalidIndexconfig, get_indexconfig
 from .readonly import ensure_deeply_readonly
 from .log import threadlog
 
@@ -35,8 +37,6 @@ class IndexParser:
     def __init__(self, project):
         self.project = normalize_name(project)
         self.basename2link = {}
-        self.crawllinks = set()
-        self.egglinks = []
 
     def _mergelink_ifbetter(self, newlink):
         """
@@ -61,32 +61,15 @@ class IndexParser:
         """ return sorted releaselinks list """
         l = sorted(map(BasenameMeta, self.basename2link.values()),
                    reverse=True)
-        return self.egglinks + [x.obj for x in l]
+        return [x.obj for x in l]
 
-    def parse_index(self, disturl, html, scrape=True):
+    def parse_index(self, disturl, html):
         p = HTMLPage(html, disturl.url)
         seen = set()
         for link in p.links:
             newurl = Link(link.url, requires_python=link.requires_python)
             if not newurl.is_valid_http_url():
                 continue
-            eggfragment = newurl.eggfragment
-            if scrape and eggfragment:
-                if not normalize_name(eggfragment).startswith(self.project):
-                    threadlog.debug("skip egg link %s (project: %s)",
-                              newurl, self.project)
-                    continue
-                if newurl.basename:
-                    # XXX seems we have to maintain a particular
-                    # order to keep pip/easy_install happy with some
-                    # packages (e.g. nose)
-                    if newurl not in self.egglinks:
-                        self.egglinks.insert(0, newurl)
-                else:
-                    threadlog.warn("cannot handle egg directory link (svn?) "
-                                   "skipping: %s (project: %s)",
-                                   newurl, self.project)
-                continue
             if is_archive_of_project(newurl, self.project):
                 if not newurl.is_valid_http_url():
                     threadlog.warn("unparseable/unsupported url: %r", newurl)
@@ -94,58 +77,77 @@ class IndexParser:
                     seen.add(newurl.url)
                     self._mergelink_ifbetter(newurl)
                     continue
-        if scrape:
-            for link in p.rel_links():
-                if link.url not in seen:
-                    disturl = URL(link.url)
-                    if disturl.is_valid_http_url():
-                        self.crawllinks.add(disturl)
 
-def parse_index(disturl, html, scrape=True):
+
+def parse_index(disturl, html):
     if not isinstance(disturl, URL):
         disturl = URL(disturl)
     project = disturl.basename or disturl.parentbasename
     parser = IndexParser(project)
-    parser.parse_index(disturl, html, scrape=scrape)
+    parser.parse_index(disturl, html)
     return parser
 
 
-def perform_crawling(pypistage, result, numthreads=10):
-    pending = set(result.crawllinks)
-    while pending:
-        try:
-            crawlurl = pending.pop()
-        except KeyError:
-            break
-        threadlog.info("visiting crawlurl %s", crawlurl)
-        response = pypistage.httpget(crawlurl.url, allow_redirects=True)
-        threadlog.info("crawlurl %s %s", crawlurl, response)
-        assert hasattr(response, "status_code")
-        if not isinstance(response, int) and response.status_code == 200:
-            ct = response.headers.get("content-type", "").lower()
-            if ct.startswith("text/html"):
-                result.parse_index(
-                    URL(response.url), response.text, scrape=False)
-                continue
-        threadlog.warn("crawlurl %s status %s", crawlurl, response)
-
-
 class PyPIStage(BaseStage):
-    def __init__(self, xom, username, index, ixconfig):
-        super(PyPIStage, self).__init__(xom, username, index, ixconfig)
+    def __init__(self, xom, username, index, ixconfig, customizer_cls):
+        super(PyPIStage, self).__init__(
+            xom, username, index, ixconfig, customizer_cls)
         self.httpget = self.xom.httpget  # XXX is requests/httpget multi-thread safe?
-        self.cache_expiry = self.ixconfig.get(
-            'mirror_cache_expiry', xom.config.args.mirror_cache_expiry)
         self.xom = xom
-        self.offline = self.xom.config.args.offline_mode
+        self.offline = self.xom.config.offline_mode
+        self.timeout = xom.config.request_timeout
         # list of locally mirrored projects
         self.key_projects = self.keyfs.PROJNAMES(user=username, index=index)
-        if xom.is_replica():
-            url = xom.config.master_url
-            self.mirror_url = url.joinpath("%s/+simple/" % self.name).url
+        # used to log about stale projects only once
+        self._offline_logging = set()
+
+    @property
+    def cache_expiry(self):
+        return self.ixconfig.get(
+            'mirror_cache_expiry', self.xom.config.mirror_cache_expiry)
+
+    @property
+    def mirror_url(self):
+        if self.xom.is_replica():
+            url = self.xom.config.master_url
+            return url.joinpath("%s/+simple/" % self.name).url
         else:
             url = URL(self.ixconfig['mirror_url'])
-            self.mirror_url = url.asdir().url
+            return url.asdir().url
+
+    @property
+    def use_external_url(self):
+        return self.ixconfig.get('mirror_use_external_urls', False)
+
+    def get_possible_indexconfig_keys(self):
+        return tuple(dict(self.get_default_config_items())) + (
+            "custom_data", "description",
+            "mirror_url", "mirror_cache_expiry",
+            "mirror_use_external_urls",
+            "mirror_web_url_fmt", "title")
+
+    def get_default_config_items(self):
+        return [("volatile", True)]
+
+    def normalize_indexconfig_value(self, key, value):
+        if key == "volatile":
+            return ensure_boolean(value)
+        if key == "mirror_url":
+            if not value.startswith(("http://", "https://")):
+                raise self.InvalidIndexconfig([
+                    "'mirror_url' option must be a URL."])
+            return value
+        if key == "mirror_cache_expiry":
+            try:
+                value = int(value)
+            except (TypeError, ValueError):
+                raise self.InvalidIndexconfig([
+                    "'mirror_cache_expiry' option must be an integer"])
+            return value
+        if key == "mirror_use_external_urls":
+            return ensure_boolean(value)
+        if key in ("custom_data", "description", "mirror_web_url_fmt", "title"):
+            return value
 
     def delete(self):
         # delete all projects on this index
@@ -154,21 +156,6 @@ class PyPIStage(BaseStage):
         self.key_projects.delete()
         BaseStage.delete(self)
 
-    def modify(self, index=None, **kw):
-        if 'type' in kw and self.ixconfig["type"] != kw['type']:
-            raise InvalidIndexconfig(
-                ["the 'type' of an index can't be changed"])
-        kw.pop("type", None)
-        ixconfig = get_indexconfig(
-            self.xom.config.hook, type=self.ixconfig["type"], **kw)
-        # modify user/indexconfig
-        with self.user.key.update() as userconfig:
-            newconfig = userconfig["indexes"][self.index]
-            newconfig.update(ixconfig)
-            threadlog.info("modified index %s: %s", self.name, ixconfig)
-            self.ixconfig = newconfig
-            return newconfig
-
     def add_project_name(self, project):
         project = normalize_name(project)
         projects = self.key_projects.get(readonly=False)
@@ -226,9 +213,9 @@ class PyPIStage(BaseStage):
 
     @property
     def cache_projectnames(self):
-        """ filesystem-persistent cache for full list of projectnames. """
+        """ cache for full list of projectnames. """
         # we could keep this info inside keyfs but pypi.org
-        # produces a 3MB list of names and it changes often which
+        # produces a several MB list of names and it changes often which
         # would spam the database.
         try:
             return self.xom.get_singleton(self.name, "projectnames")
@@ -251,7 +238,16 @@ class PyPIStage(BaseStage):
 
     def _get_remote_projects(self):
         headers = {"Accept": "text/html"}
-        response = self.httpget(self.mirror_url, allow_redirects=True, extra_headers=headers)
+        # use a minimum of 30 seconds as timeout for remote server and
+        # 60s when running as replica, because the list can be quite large
+        # and the master might take a while to process it
+        if self.xom.is_replica():
+            timeout = max(self.timeout, 60)
+        else:
+            timeout = max(self.timeout, 30)
+        response = self.httpget(
+            self.mirror_url, allow_redirects=True, extra_headers=headers,
+            timeout=timeout)
         if response.status_code != 200:
             raise self.UpstreamError("URL %r returned %s %s",
                 self.mirror_url, response.status_code, response.reason)
@@ -294,10 +290,17 @@ class PyPIStage(BaseStage):
 
                     # trigger an initial-load event on master
                     if not self.xom.is_replica():
+                        # make sure we are at the current serial
+                        # this avoids setting the value again when
+                        # called from the notification thread
+                        self.keyfs.restart_read_transaction()
                         k = self.keyfs.MIRRORNAMESINIT(user=self.username, index=self.index)
                         if k.get() == 0:
                             self.keyfs.restart_as_write_transaction()
                             k.set(1)
+                else:
+                    # mark current without updating contents
+                    self.cache_projectnames.mark_current()
 
         return projects
 
@@ -309,7 +312,7 @@ class PyPIStage(BaseStage):
         assert links != ()  # we don't store the old "Not Found" marker anymore
         assert isinstance(serial, int)
         assert project == normalize_name(project), project
-        data = {"serial": serial, "links": links, 
+        data = {"serial": serial, "links": links,
             "requires_python": requires_python}
         key = self.key_projsimplelinks(project)
         old = key.get()
@@ -358,9 +361,8 @@ class PyPIStage(BaseStage):
         threadlog.debug("cleared cache for %s", project)
 
     def get_simplelinks_perstage(self, project):
-        """ return all releaselinks from the index and referenced scrape
-        pages, returning cached entries if we have a recent enough
-        request stored locally.
+        """ return all releaselinks from the index, returning cached entries
+        if we have a recent enough request stored locally.
 
         Raise UpstreamError if the pypi server cannot be reached or
         does not return a fresh enough page although we know it must
@@ -370,7 +372,11 @@ class PyPIStage(BaseStage):
         is_expired, links, cache_serial = self._load_cache_links(project)
         if self.offline and links is None:
             raise self.UpstreamError("offline mode")
-        if not is_expired:
+        if self.offline or not is_expired:
+            if self.offline and project not in self._offline_logging:
+                threadlog.debug(
+                    "using stale links for %r due to offline mode", project)
+                self._offline_logging.add(project)
             return links
 
         is_retrieval_expired = self.cache_retrieve_times.is_expired(
@@ -383,7 +389,7 @@ class PyPIStage(BaseStage):
         # get the simple page for the project
         url = self.mirror_url + project + "/"
         threadlog.debug("reading index %s", url)
-        response = self.httpget(url, allow_redirects=True)
+        response = self.httpget(url, allow_redirects=True, timeout=self.timeout)
         if response.status_code != 200:
             # if we have and old result, return it. While this will
             # miss the rare event of actual project deletions it allows
@@ -428,11 +434,9 @@ class PyPIStage(BaseStage):
         ret_project = response.url.strip("/").split("/")[-1]
         assert project == normalize_name(ret_project)
 
-
-        # parse simple index's link and perform crawling
+        # parse simple index's link
         assert response.text is not None, response.text
         result = parse_index(response.url, response.text)
-        perform_crawling(self, result)
         releaselinks = list(result.releaselinks)
 
         # first we try to process mirror links without an explicit write transaction.
@@ -473,12 +477,12 @@ class PyPIStage(BaseStage):
             devpi_serial = int(response.headers["X-DEVPI-SERIAL"])
             threadlog.debug("get_simplelinks pypi: waiting for devpi_serial %r",
                             devpi_serial)
-            self.keyfs.wait_tx_serial(devpi_serial)
-            threadlog.debug("get_simplelinks pypi: finished waiting for devpi_serial %r",
-                            devpi_serial)
-            # XXX raise TransactionRestart to get a consistent clean view
-            self.keyfs.restart_read_transaction()
-            is_expired, links, cache_serial = self._load_cache_links(project)
+            if self.keyfs.wait_tx_serial(devpi_serial, timeout=self.timeout):
+                threadlog.debug("get_simplelinks pypi: finished waiting for devpi_serial %r",
+                                devpi_serial)
+                # XXX raise TransactionRestart to get a consistent clean view
+                self.keyfs.restart_read_transaction()
+                is_expired, links, cache_serial = self._load_cache_links(project)
             if links is not None:
                 self.cache_retrieve_times.refresh(project)
                 return links
@@ -500,20 +504,35 @@ class PyPIStage(BaseStage):
 
     def list_versions_perstage(self, project):
         try:
-            return set(x.get_eggfragment_or_version()
+            return set(x.version
                        for x in map(SimplelinkMeta, self.get_simplelinks_perstage(project)))
         except self.UpstreamNotFoundError:
             return []
 
+    def get_last_project_change_serial_perstage(self, project, at_serial=None):
+        tx = self.keyfs.tx
+        if at_serial is None:
+            at_serial = tx.at_serial
+        info = tx.get_last_serial_and_value_at(
+            self.key_projsimplelinks(project),
+            at_serial, raise_on_error=False)
+        if info is None:
+            # never existed
+            return -1
+        (last_serial, links) = info
+        return last_serial
+
     def get_versiondata_perstage(self, project, version, readonly=True):
         project = normalize_name(project)
         verdata = {}
         for sm in map(SimplelinkMeta, self.get_simplelinks_perstage(project)):
-            link_version = sm.get_eggfragment_or_version()
+            link_version = sm.version
             if version == link_version:
                 if not verdata:
                     verdata['name'] = project
                     verdata['version'] = version
+                if sm.require_python is not None:
+                    verdata['requires_python'] = sm.require_python
                 elinks = verdata.setdefault("+elinks", [])
                 entrypath = sm._url.path
                 elinks.append({"rel": "releasefile", "entrypath": entrypath})
@@ -522,6 +541,17 @@ class PyPIStage(BaseStage):
         return verdata
 
 
+class PyPICustomizer(BaseStageCustomizer):
+    pass
+
+
+@hookimpl
+def devpiserver_get_stage_customizer_classes():
+    # prevent plugins from installing their own under the reserved names
+    return [
+        ("mirror", PyPICustomizer)]
+
+
 class ProjectNamesCache:
     """ Helper class for maintaining project names from a mirror. """
     def __init__(self):
@@ -543,9 +573,12 @@ class ProjectNamesCache:
         return self._data
 
     def set(self, data):
-        """ Set data, updating timestamp and writing to local file"""
+        """ Set data and update timestamp. """
         if data is not self._data:
             self._data = data.copy()
+        self.mark_current()
+
+    def mark_current(self):
         self._timestamp = time.time()
 
 
