@@ -97,7 +97,7 @@ class Connection(BaseConnection):
         q = "INSERT OR REPLACE INTO files (path, size, data) VALUES (?, ?, ?)"
         c.execute(q, (path, len(content), sqlite3.Binary(content)))
         c.close()
-        self.dirty_files[path] = content
+        self.dirty_files[path] = True
 
     def io_file_open(self, path):
         return py.io.BytesIO(self.io_file_get(path))
@@ -138,10 +138,10 @@ class Connection(BaseConnection):
         self.commit()
 
 
-class BaseStorage:
+class BaseStorage(object):
     def __init__(self, basedir, notify_on_commit, cache_size):
         self.basedir = basedir
-        self.sqlpath = self.basedir.join(".sqlite")
+        self.sqlpath = self.basedir.join(self.db_filename)
         self._notify_on_commit = notify_on_commit
         self._changelog_cache = LRUCache(cache_size)  # is thread safe
         self.last_commit_timestamp = time.time()
@@ -233,6 +233,7 @@ class BaseStorage:
 
 class Storage(BaseStorage):
     Connection = Connection
+    db_filename = ".sqlite_db"
 
     def perform_crash_recovery(self):
         pass
@@ -274,6 +275,25 @@ def devpiserver_storage_backend(settings):
         description="SQLite backend with files in DB for testing only")
 
 
+@hookimpl
+def devpiserver_metrics(request):
+    result = []
+    xom = request.registry["xom"]
+    storage = xom.keyfs._storage
+    if not isinstance(storage, BaseStorage):
+        return result
+    cache = getattr(storage, '_changelog_cache', None)
+    if cache is None:
+        return result
+    result.extend([
+        ('devpi_server_storage_cache_evictions', 'counter', cache.evictions),
+        ('devpi_server_storage_cache_hits', 'counter', cache.hits),
+        ('devpi_server_storage_cache_lookups', 'counter', cache.lookups),
+        ('devpi_server_storage_cache_misses', 'counter', cache.misses),
+        ('devpi_server_storage_cache_size', 'gauge', cache.size)])
+    return result
+
+
 class Writer:
     def __init__(self, storage, conn):
         self.conn = conn
@@ -281,13 +301,14 @@ class Writer:
         self.changes = {}
         self.next_serial = conn.last_changelog_serial + 1
 
-    def record_set(self, typedkey, value=None):
+    def record_set(self, typedkey, value=None, back_serial=None):
         """ record setting typedkey to value (None means it's deleted) """
         assert not isinstance(value, ReadonlyView), value
-        try:
-            _, back_serial = self.conn.db_read_typedkey(typedkey.relpath)
-        except KeyError:
-            back_serial = -1
+        if back_serial is None:
+            try:
+                _, back_serial = self.conn.db_read_typedkey(typedkey.relpath)
+            except KeyError:
+                back_serial = -1
         self.conn.db_write_typedkey(typedkey.relpath, typedkey.name, self.next_serial)
         # at __exit__ time we write out changes to the _changelog_cache
         # so we protect here against the caller modifying the value later
