@@ -5,11 +5,11 @@ recursive cache of pypi.org packages.
 """
 from __future__ import unicode_literals
 import inspect
-import os, sys
+import os
 import py
+import sys
 
 from requests import Response, exceptions
-from devpi_common.metadata import Version
 from devpi_common.types import cached_property
 from devpi_common.request import new_requests_session
 from .config import parseoptions, get_pluginmanager
@@ -26,40 +26,90 @@ class Fatal(Exception):
 def fatal(msg):
     raise Fatal(msg)
 
-def check_compatible_version(xom):
-    args = xom.config.args
-    if args.export:
+
+DATABASE_VERSION = "4"
+
+
+def check_compatible_version(config):
+    if not config.serverdir.exists():
         return
-    state_version = xom.get_state_version()
+    state_version = get_state_version(config)
     if server_version != state_version:
         state_ver = tuple(state_version.split("."))
-        server_ver = tuple(server_version.split("."))
-        incompatible = (
-            state_ver[:1] != server_ver[:1] or
-            Version(state_version) < Version("2.2.dev0"))
-        if incompatible:
+        if state_ver[0] != DATABASE_VERSION:
             fatal("Incompatible state: server %s cannot run serverdir "
-                  "%s created by %s.\n"
-                  "Use --export from older version, then --import with newer "
-                  "version."
-                  %(server_version, xom.config.serverdir, state_version))
+                  "%s created at database version %s.\n"
+                  "Use devpi-export (or --export) from older version, then "
+                  "devpi-import with newer version."
+                  % (server_version, config.serverdir, state_ver[0]))
 
-        xom.set_state_version(server_version)
-        tw = py.io.TerminalWriter(sys.stderr)
-        tw.line("minor version upgrade: setting serverstate to %s from %s" %(
-                server_version, state_version), bold=True)
+
+def get_state_version(config):
+    versionfile = config.serverdir.join(".serverversion")
+    if not versionfile.exists():
+        fatal(
+            "serverdir %s is non-empty and misses devpi-server meta information. "
+            "You need to specify an empty directory or a directory that was "
+            "previously managed by devpi-server>=1.2" % config.serverdir)
+    return versionfile.read()
+
+
+def set_state_version(config, version=DATABASE_VERSION):
+    versionfile = config.serverdir.join(".serverversion")
+    versionfile.dirpath().ensure(dir=1)
+    versionfile.write(version)
+
+
+def check_python_version():
+    if sys.version_info.major < 3:
+        import warnings
+        warnings.warn(
+            "DEPRECATION: Python 2.7 will reach the end of its life on "
+            "January 1st, 2020. Please upgrade your Python as Python 2.7 "
+            "won't be maintained after that date.\n"
+            "The recommended way for upgrading devpi-server to Python 3.x "
+            "is to first use the --export option with your existing "
+            "Python 2.7 setup. Then use --import in a new Python 3.x "
+            "environment.")
 
 
 def main(argv=None):
     """ devpi-server command line entry point. """
+    check_python_version()
     pluginmanager = get_pluginmanager()
     try:
         return _main(pluginmanager, argv=argv)
     except Fatal as e:
         tw = py.io.TerminalWriter(sys.stderr)
-        tw.line("fatal: %s" %  e.args[0], red=True)
+        tw.line("fatal: %s" % e.args[0], red=True)
         return 1
 
+
+def xom_from_config(config):
+    check_compatible_version(config)
+
+    # read/create node UUID and role of this server
+    config.init_nodeinfo()
+
+    if config.sqlite_file_needed_but_missing():
+        fatal(
+            "No sqlite storage found in serverdir."
+            " Or you need to run with --storage to specify the storage type,"
+            " or you first need to run devpi-init or devpi-import"
+            " in order to create the sqlite database."
+        )
+
+    return XOM(config)
+
+
+def init_default_indexes(xom):
+    # we deliberately call get_current_serial first to establish a connection
+    # to the backend and in case of sqlite create the database
+    if xom.keyfs.get_current_serial() == -1 and not xom.is_replica():
+        with xom.keyfs.transaction(write=True):
+            set_default_indexes(xom.model)
+
+
 def _main(pluginmanager, argv=None):
     # During parsing of options logging should not be used
 
@@ -77,25 +127,35 @@ def _main(pluginmanager, argv=None):
 
     if args.genconfig:
         from devpi_server.genconfig import genconfig
-        return genconfig(config, argv[1:])
+        import warnings
+        warnings.warn(
+            "DEPRECATION: the --gen-config option is deprecated, use the "
+            "devpi-gen-config command instead")
+        return genconfig(config, argv)
 
+    # now we can configure logging
     configure_logging(config.args)
 
     if args.init:
+        import warnings
+        warnings.warn(
+            "DEPRECATION: the --init option is deprecated, use the "
+            "devpi-init command instead")
         if config.path_nodeinfo.exists():
             fatal("The path '%s' already contains devpi-server data." % config.serverdir)
     elif not args.import_:
         if not config.path_nodeinfo.exists():
-            fatal("The path '%s' contains no devpi-server data, use --init to initialize." % config.serverdir)
+            fatal("The path '%s' contains no devpi-server data, use devpi-init to initialize." % config.serverdir)
 
-    # read/create node UUID and role of this server
-    config.init_nodeinfo()
+    if args.init or args.import_:
+        sdir = config.serverdir
+        if not (sdir.exists() and len(sdir.listdir()) >= 2):
+            set_state_version(config, DATABASE_VERSION)
 
-    xom = XOM(config)
-    if not xom.is_replica() and xom.keyfs.get_current_serial() == -1:
-        with xom.keyfs.transaction(write=True):
-            set_default_indexes(xom.model)
-    check_compatible_version(xom)
+    xom = xom_from_config(config)
+
+    if args.init:
+        init_default_indexes(xom)
 
     if args.start or args.stop or args.log or args.status:
         xprocdir = config.serverdir.join(".xproc")
@@ -118,6 +178,10 @@ def _main(pluginmanager, argv=None):
 
     if args.passwd:
         from devpi_server.model import run_passwd
+        import warnings
+        warnings.warn(
+            "DEPRECATION: the --passwd option is deprecated, use the "
+            "devpi-passwd command instead")
         with xom.keyfs.transaction(write=True):
             return run_passwd(xom.model, config.args.passwd)
 
@@ -136,8 +200,21 @@ def wsgi_run(xom, app):
     from waitress import serve
     host = xom.config.args.host
     port = xom.config.args.port
-    threads = xom.config.args.threads
-    max_body = xom.config.args.max_request_body_size
+    unix_socket = xom.config.args.unix_socket
+    kwargs = dict(
+        threads=xom.config.args.threads,
+        max_request_body_size=xom.config.args.max_request_body_size)
+    if unix_socket is not None:
+        kwargs['unix_socket'] = unix_socket
+        if xom.config.args.unix_socket_perms is not None:
+            kwargs['unix_socket_perms'] = xom.config.args.unix_socket_perms
+        if host == 'localhost':
+            host = None
+        if port == 3141:
+            port = None
+    if host or port:
+        kwargs['host'] = host
+        kwargs['port'] = port
     log = xom.log
     log.info("devpi-server version: %s", server_version)
     log.info("serverdir: %s" % xom.config.serverdir)
@@ -145,7 +222,7 @@ def wsgi_run(xom, app):
     hostaddr = "http://%s:%s" % (host, port)
     hostaddr6 = "http://[%s]:%s" % (host, port)
     log.info("serving at url: %s (might be %s for IPv6)", hostaddr, hostaddr6)
-    log.info("using %s threads", threads)
+    log.info("using %s threads", kwargs['threads'])
     log.info("bug tracker: https://github.com/devpi/devpi/issues")
     log.info("IRC: #devpi on irc.freenode.net")
     if "WEBTRACE" in os.environ and xom.config.args.debug:
@@ -153,7 +230,7 @@ def wsgi_run(xom, app):
         app = make_eval_exception(app, {})
     try:
         log.info("Hit Ctrl-C to quit.")
-        serve(app, host=host, port=port, threads=threads, max_request_body_size=max_body)
+        serve(app, **kwargs)
     except KeyboardInterrupt:
         pass
     return 0
@@ -179,27 +256,10 @@ class XOM:
         self.thread_pool = mythread.ThreadPool()
         if httpget is not None:
             self.httpget = httpget
-        sdir = config.serverdir
-        if not (sdir.exists() and len(sdir.listdir()) >= 2):
-            self.set_state_version(server_version)
         self.log = threadlog
         self.polling_replicas = {}
         self._stagecache = {}
 
-    def get_state_version(self):
-        versionfile = self.config.serverdir.join(".serverversion")
-        if not versionfile.exists():
-            fatal(
-            "serverdir %s is non-empty and misses devpi-server meta information. "
-            "You need to specify an empty directory or a directory that was "
-            "previously managed by devpi-server>=1.2" % self.config.serverdir)
-        return versionfile.read()
-
-    def set_state_version(self, version):
-        versionfile = self.config.serverdir.join(".serverversion")
-        versionfile.dirpath().ensure(dir=1)
-        versionfile.write(version)
-
     def get_singleton(self, indexpath, key):
         """ return a per-xom singleton for the given indexpath and key
         or raise KeyError if no such singleton was set yet.
@@ -235,11 +295,19 @@ class XOM:
         args = xom.config.args
         if args.export:
             from devpi_server.importexport import do_export
+            import warnings
+            warnings.warn(
+                "DEPRECATION: the --export option is deprecated, use the "
+                "devpi-export command instead")
             #xom.thread_pool.start_one(xom.keyfs.notifier)
             return do_export(args.export, xom)
 
         if args.import_:
             from devpi_server.importexport import do_import
+            import warnings
+            warnings.warn(
+                "DEPRECATION: the --import option is deprecated, use the "
+                "devpi-import command instead")
             # we need to start the keyfs notifier so that import
             # can wait on completion of events
             if args.wait_for_events:
@@ -268,7 +336,7 @@ class XOM:
     @cached_property
     def filestore(self):
         from devpi_server.filestore import FileStore
-        return FileStore(self)
+        return FileStore(self.keyfs)
 
     @cached_property
     def keyfs(self):
@@ -280,21 +348,27 @@ class XOM:
             readonly=self.is_replica(),
             cache_size=self.config.args.keyfs_cache_size)
         add_keys(self, keyfs)
-        if not self.config.args.requests_only:
+        try:
+            keyfs.finalize_init()
+        except Exception:
+            threadlog.exception("Error while trying to initialize storage")
+            fatal("Couldn't initialize storage")
+        if not self.config.requests_only:
             self.thread_pool.register(keyfs.notifier)
         return keyfs
 
     def new_http_session(self, component_name, max_retries=None):
         session = new_requests_session(agent=(component_name, server_version), max_retries=max_retries)
-        session.cert = self.config.args.replica_cert
+        session.cert = self.config.replica_cert
         return session
 
     @cached_property
     def _httpsession(self):
-        return self.new_http_session("server", max_retries=self.config.args.replica_max_retries)
+        max_retries = self.config.replica_max_retries
+        return self.new_http_session("server", max_retries=max_retries)
 
     def httpget(self, url, allow_redirects, timeout=None, extra_headers=None):
-        if self.config.args.offline_mode:
+        if self.config.offline_mode:
             resp = Response()
             resp.status_code = 503  # service unavailable
             return resp
@@ -311,15 +385,35 @@ class XOM:
         except OSError:
             location = get_caller_location()
             threadlog.warn("OS error during httpget of %s at %s", url, location)
-            return FatalResponse(repr(sys.exc_info()[1]))
+            return FatalResponse(url, repr(sys.exc_info()[1]))
         except exceptions.ConnectionError:
             location = get_caller_location()
             threadlog.warn("Connection error during httpget of %s at %s", url, location)
-            return FatalResponse(repr(sys.exc_info()[1]))
+            return FatalResponse(url, repr(sys.exc_info()[1]))
         except self._httpsession.Errors:
             location = get_caller_location()
             threadlog.warn("HTTPError during httpget of %s at %s", url, location)
-            return FatalResponse(repr(sys.exc_info()[1]))
+            return FatalResponse(url, repr(sys.exc_info()[1]))
+
+    def view_deriver(self, view, info):
+        if self.is_replica():
+            if info.options.get('is_mutating', True):
+                from .model import ensure_list
+                from .replica import proxy_view_to_master
+                from .views import is_mutating_http_method
+                request_methods = info.options['request_method']
+                if request_methods is None:
+                    request_methods = []
+                for request_method in ensure_list(request_methods):
+                    if is_mutating_http_method(request_method):
+                        # we got a view which uses a mutating method and isn't
+                        # marked to be excluded, so we replace the view with
+                        # one that proxies to the master, because a replica
+                        # must not modify its database except via the
+                        # replication protocol
+                        return proxy_view_to_master
+        return view
+    view_deriver.options = ('is_mutating',)
 
     def create_app(self):
         from devpi_server.view_auth import DevpiAuthenticationPolicy
@@ -329,6 +423,7 @@ class XOM:
         from pkg_resources import get_distribution
         from pyramid.authorization import ACLAuthorizationPolicy
         from pyramid.config import Configurator
+        from pyramid.viewderivers import INGRESS
         log = self.log
         log.debug("creating application in process %s", os.getpid())
         pyramid_config = Configurator(root_factory='devpi_server.view_auth.RootFactory')
@@ -345,14 +440,33 @@ class XOM:
                 version_info.append(key)
         version_info.sort()
         pyramid_config.registry['devpi_version_info'] = version_info
+        pyramid_config.registry['xom'] = self
+        index_classes = {}
+        customizer_classes = sum(
+            self.config.hook.devpiserver_get_stage_customizer_classes(),
+            [])
+        for ixtype, ixclass in customizer_classes:
+            index_classes.setdefault(ixtype, []).append(ixclass)
+        for ixtype, ixclasses in index_classes.items():
+            if len(ixclasses) > 1:
+                fatal(
+                    "multiple implementation classes for index type '%s':\n%s"
+                    % (
+                        ixtype,
+                        "\n".join(
+                            "%s.%s" % (x.__module__, x.__name__)
+                            for x in ixclasses)))
         self.config.hook.devpiserver_pyramid_configure(
                 config=self.config,
                 pyramid_config=pyramid_config)
 
+        pyramid_config.add_view_deriver(self.view_deriver, under=INGRESS)
         pyramid_config.add_view_predicate('content_type', ContentTypePredicate)
 
         pyramid_config.add_route("/+changelog/{serial}",
-                                 "/+changelog/{serial}")
+                                 r"/+changelog/{serial:\d+}")
+        pyramid_config.add_route("/+changelog/{serial}-",
+                                 r"/+changelog/{serial:\d+}-")
         pyramid_config.add_route("/+status", "/+status")
         pyramid_config.add_route("/+api", "/+api", accept="application/json")
         pyramid_config.add_route("{path:.*}/+api", "{path:.*}/+api", accept="application/json")
@@ -383,15 +497,9 @@ class XOM:
 
         # register tweens for logging, transaction and replication
         pyramid_config.add_tween("devpi_server.views.tween_request_logging")
-        if self.is_replica():
-            pyramid_config.add_tween(
-                "devpi_server.replica.tween_replica_proxy",
-                over="devpi_server.views.tween_keyfs_transaction",
-                under="devpi_server.views.tween_request_logging",
-            )
-        pyramid_config.add_tween("devpi_server.views.tween_keyfs_transaction",
-            under="devpi_server.views.tween_request_logging"
-        )
+        pyramid_config.add_tween(
+            "devpi_server.views.tween_keyfs_transaction",
+            under="devpi_server.views.tween_request_logging")
         if self.config.args.profile_requests:
             pyramid_config.add_tween("devpi_server.main.tween_request_profiling")
         pyramid_config.add_request_method(get_remote_ip)
@@ -403,16 +511,20 @@ class XOM:
         pyramid_config.add_request_method(route_url)
         # XXX end hack
         pyramid_config.scan()
-        pyramid_config.registry['xom'] = self
         app = pyramid_config.make_wsgi_app()
         if self.is_replica():
             from devpi_server.replica import ReplicaThread, register_key_subscribers
+            search_path = self.config.args.replica_file_search_path
+            if search_path and not os.path.exists(search_path):
+                fatal(
+                    "search path for existing replica files doesn't "
+                    "exist: %s" % search_path)
             register_key_subscribers(self)
             self.replica_thread = ReplicaThread(self)
             # the replica thread replays keyfs changes
             # and project-specific changes are discovered
             # and replayed through the PypiProjectChange event
-            if not self.config.args.requests_only:
+            if not self.config.requests_only:
                 self.thread_pool.register(self.replica_thread)
         return OutsideURLMiddleware(app, self)
 
@@ -426,7 +538,8 @@ class XOM:
 class FatalResponse:
     status_code = -1
 
-    def __init__(self, reason):
+    def __init__(self, url, reason):
+        self.url = url
         self.reason = reason
 
     def __repr__(self):
@@ -470,7 +583,10 @@ def apifatal(request, *args, **kwargs):
 def set_default_indexes(model):
     root_user = model.get_user("root")
     if not root_user:
-        root_user = model.create_user("root", "")
+        root_user = model.create_user(
+            "root",
+            model.xom.config.args.root_passwd,
+            pwhash=model.xom.config.args.root_passwd_hash)
         threadlog.info("created root user")
     userconfig = root_user.key.get(readonly=False)
     indexes = userconfig["indexes"]
@@ -479,6 +595,7 @@ def set_default_indexes(model):
         root_user.key.set(userconfig)
         threadlog.info("created root/pypi index")
 
+
 _pypi_ixconfig_default = {
     "type": "mirror", "volatile": False,
     "title": "PyPI",
