@@ -1,3 +1,5 @@
+# this file is shared via symlink with devpi-client,
+# so for the time being it must continue to work with Python 2
 import pytest
 
 try:
@@ -122,12 +124,14 @@ class TestIndexThings:
         data = mapp.getjson("/root/pypi")
         res = data["result"]
         res.pop("projects")
-        assert res == {
-            "type": "mirror",
-            "volatile": False,
-            "title": "PyPI",
-            "mirror_url": "https://pypi.org/simple/",
-            "mirror_web_url_fmt": "https://pypi.org/project/{name}/"}
+        assert sorted(res.keys()) == sorted([
+            "type", "volatile", "title", "mirror_url", "mirror_web_url_fmt"])
+        assert res["type"] == "mirror"
+        assert res["volatile"] is False
+        assert res["title"] == "PyPI"
+        assert 'pypi' in res["mirror_url"]
+        assert 'pypi' in res["mirror_web_url_fmt"]
+        assert '{name}' in res["mirror_web_url_fmt"]
 
     def test_create_index_base_empty(self, mapp):
         indexconfig = dict(bases="")
@@ -197,7 +201,7 @@ class TestIndexThings:
     def test_create_index_and_config_gets(self, mapp):
         mapp.create_and_login_user("cuser2")
         mapp.create_index("dev")
-        res =  mapp.getjson("/cuser2/dev")
+        res = mapp.getjson("/cuser2/dev")
         assert res["type"] == "indexconfig"
         assert res["result"]["projects"] == []
 
@@ -230,32 +234,53 @@ class TestIndexThings:
         assert res["result"]["title"] == "foo"
         assert res["result"]["description"] == "bar"
 
-    def test_whitelist_setting(self, mapp):
+    def test_whitelist_setting(self, mapp, server_version):
+        from pkg_resources import parse_version
+        # pypi_whilelist was removed in 5.0.0
+        pypi_whitelist_version = parse_version("5dev")
         mapp.create_and_login_user("cuser7")
         mapp.create_index("dev")
         mapp.use("cuser7/dev")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == []
         mapp.set_mirror_whitelist("foo")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == ['foo']
         mapp.set_mirror_whitelist("foo,bar")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == ['foo', 'bar']
         mapp.set_mirror_whitelist("he_llo")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == ['he-llo']
         mapp.set_mirror_whitelist("he_llo,Django")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == ['he-llo', 'django']
         mapp.set_mirror_whitelist("*")
         res = mapp.getjson("/cuser7/dev")['result']
-        assert res['pypi_whitelist'] == []
+        if server_version < pypi_whitelist_version:
+            assert res['pypi_whitelist'] == []
+        else:
+            assert 'pypi_whitelist' not in res
         assert res['mirror_whitelist'] == ['*']
 
 
@@ -446,7 +471,11 @@ class TestMirrorIndexThings:
         r = mapp.get_simple("pkg")
         assert b'ed7/002b439e9ac84/pkg-1.0.tar.gz' in r.body
 
-    def test_releases_urlquoting(self, mapp, simpypi):
+    def test_releases_urlquoting(self, mapp, server_version, simpypi):
+        from pkg_resources import parse_version
+        quoting_devpi_version = parse_version("4.3.1dev")
+        if server_version < quoting_devpi_version:
+            pytest.skip("devpi-server without mirror url quoting fix")
         mapp.create_and_login_user('mirror9')
         indexconfig = dict(
             type="mirror",
@@ -462,7 +491,11 @@ class TestMirrorIndexThings:
         assert len(result) == 1
         assert result[0].endswith('/mirror9/mirror/+e/%s_pkg/pkg-1!2017.4+devpi.zip' % base)
 
-    def test_releases_urlquoting_hash(self, mapp, simpypi):
+    def test_releases_urlquoting_hash(self, mapp, server_version, simpypi):
+        from pkg_resources import parse_version
+        quoting_devpi_version = parse_version("4.3.1dev")
+        if server_version < quoting_devpi_version:
+            pytest.skip("devpi-server without mirror url quoting fix")
         mapp.create_and_login_user('mirror10')
         indexconfig = dict(
             type="mirror",
