@@ -51,36 +51,65 @@ class TestParser:
         assert isinstance(p, MyArgumentParser)
 
 class TestConfig:
-    def test_parse_secret(self, tmpdir):
+    def test_parse_secret(self, caplog, tmpdir):
+        # create a secret file
         p = tmpdir.join("secret")
         secret = "qwoieuqwelkj123"
         p.write(secret)
+        # and use it
+        caplog.clear()
         config = make_config(["devpi-server", "--secretfile=%s" % p])
+        assert config.args.secretfile == str(p)
         assert config.secretfile == str(p)
         assert config.secret == secret
+        recs = caplog.getrecords(".*new random secret.*")
+        assert len(recs) == 0
+        # now check the default
+        caplog.clear()
         config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
-        assert config.secretfile == tmpdir.join(".secret")
-        config.secretfile.write(secret)
-        assert config.secret == config.secretfile.read()
+        assert config.args.secretfile is None
+        assert config.secretfile is None
+        assert config.secret != secret
+        recs = caplog.getrecords(".*new random secret.*")
+        assert len(recs) == 1
+        # remember the secret
+        prev_secret = config.secret
+        # each startup without a secret file creates a new random secret
+        caplog.clear()
+        config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
+        assert config.args.secretfile is None
+        assert config.secretfile is None
+        assert config.secret != secret
+        assert config.secret != prev_secret
+        recs = caplog.getrecords(".*new random secret.*")
+        assert len(recs) == 1
 
-    def test_generated_secret_if_not_exists(self,
-                                            xom, tmpdir, monkeypatch):
-        config = xom.config
-        secfile = tmpdir.join("secret")
-        monkeypatch.setattr(config, "secretfile", secfile)
-        assert not secfile.check()
-        assert config.secret
-        assert config.secret == config.secretfile.read()
-        assert config.secretfile == secfile
-        #recs = caplog.getrecords()
+    def test_bbb_default_secretfile_location(self, caplog, recwarn, tmpdir):
+        import warnings
+        warnings.simplefilter("always")
+        # setup secret file in old default location
+        p = tmpdir.join(".secret")
+        secret = "qwoieuqwelkj123"
+        p.write(secret)
+        caplog.clear()
+        config = make_config(["devpi-server", "--serverdir", tmpdir.strpath])
+        # the existing file should be used
+        assert config.args.secretfile is None
+        assert config.secretfile == str(p)
+        assert config.secret == secret
+        recs = caplog.getrecords(".*new random secret.*")
+        assert len(recs) == 0
+        assert len(recwarn) == 1
+        warning = recwarn.pop(Warning)
+        assert 'deprecated existing secret' in warning.message.args[0]
 
     def test_devpi_serverdir_env(self, tmpdir, monkeypatch):
-        monkeypatch.setenv("DEVPI_SERVERDIR", tmpdir)
+        monkeypatch.setenv("DEVPI_SERVERDIR", tmpdir.strpath)
         config = make_config(["devpi-server"])
         assert config.serverdir == tmpdir
 
     def test_devpiserver_serverdir_env(self, tmpdir, monkeypatch):
-        monkeypatch.setenv("DEVPISERVER_SERVERDIR", tmpdir)
+        monkeypatch.setenv("DEVPISERVER_SERVERDIR", tmpdir.strpath)
         config = make_config(["devpi-server"])
         assert config.serverdir == tmpdir
 
@@ -140,7 +169,7 @@ class TestConfig:
                              "--serverdir", str(tmpdir)])
         with pytest.raises(Fatal) as excinfo:
             config.init_nodeinfo()
-        assert "need to specify --master-url" in str(excinfo)
+        assert "need to specify --master-url" in str(excinfo.value)
 
     def test_uuid(self, tmpdir):
         config = make_config(["devpi-server", "--serverdir", str(tmpdir)])
@@ -191,23 +220,6 @@ class TestConfig:
         xom = makexom(plugins=(keyfs_sqlite, keyfs_sqlite_fs))
         assert xom.config.storage is keyfs_sqlite_fs.Storage
 
-    def test_storage_backend_persisted(self, tmpdir):
-        from devpi_server import keyfs_sqlite
-        config = make_config(["devpi-server",
-                              "--serverdir", str(tmpdir),
-                              "--storage", "sqlite_db_files"])
-        config.init_nodeinfo()
-        assert config.storage is keyfs_sqlite.Storage
-        config = make_config(["devpi-server",
-                              "--serverdir", str(tmpdir)])
-        config.init_nodeinfo()
-        assert config.storage is keyfs_sqlite.Storage
-        config = make_config(["devpi-server",
-                              "--serverdir", str(tmpdir),
-                              "--storage", "sqlite"])
-        with pytest.raises(Fatal):
-            config.init_nodeinfo()
-
     @pytest.mark.no_storage_option
     def test_storage_backend_options(self, makexom):
         class Plugin:
@@ -279,3 +291,27 @@ class TestConfigFile:
             make_config(["devpi-server", "-c", yaml_path])
         (out, err) = capsys.readouterr()
         assert "argument --port: invalid int value: 'foo'" in err
+
+    @pytest.mark.no_storage_option
+    def test_storage_backend_options(self, makexom, make_yaml_config):
+        class Plugin:
+            @hookimpl
+            def devpiserver_storage_backend(self, settings):
+                from devpi_server import keyfs_sqlite_fs
+                self.settings = settings
+                return dict(
+                    storage=keyfs_sqlite_fs.Storage,
+                    name="foo",
+                    description="Foo backend")
+        yaml_path = make_yaml_config(textwrap.dedent("""\
+            devpi-server:
+              storage:
+                name: foo
+                bar: ham"""))
+        options = ("-c", yaml_path)
+        config = make_config(("devpi-server",) + options)
+        assert isinstance(config.args.storage, dict)
+        plugin = Plugin()
+        xom = makexom(plugins=(plugin,), opts=options)
+        assert xom.config.storage_info["name"] == "foo"
+        assert plugin.settings == {"bar": "ham"}
