@@ -11,8 +11,10 @@ from devpi_server.importexport import Exporter
 from devpi_server.importexport import IndexTree
 from devpi_server.importexport import do_export, do_import
 from devpi_server.main import Fatal
+from devpi_server.readonly import get_mutable_deepcopy
 from devpi_common.archive import Archive, zip_dict
 from devpi_common.metadata import Version
+from devpi_common.url import URL
 
 import devpi_server
 
@@ -20,6 +22,7 @@ def make_export(tmpdir, xom):
     xom.config.init_nodeinfo()
     return do_export(tmpdir, xom)
 
+
 pytestmark = [pytest.mark.notransaction]
 
 
@@ -49,7 +52,7 @@ def test_has_users_or_stages(xom):
 def test_not_exists(tmpdir, xom):
     p = tmpdir.join("hello")
     with pytest.raises(Fatal):
-         do_import(p, xom)
+        do_import(p, xom)
 
 def test_import_wrong_dumpversion(tmpdir, xom):
     tmpdir.join("dataindex.json").write('{"dumpversion": "0"}')
@@ -68,15 +71,23 @@ def test_empty_export(tmpdir, xom):
         make_export(tmpdir, xom)
 
 
-def test_export_empty_serverdir(tmpdir, capfd, monkeypatch):
+@pytest.mark.parametrize("use_option", (False, True))
+def test_export_empty_serverdir(tmpdir, capfd, monkeypatch, use_option):
+    from devpi_server.importexport import export
     from devpi_server.main import main
     empty = tmpdir.join("empty").ensure(dir=True)
-    export = tmpdir.join("export")
+    export_dir = tmpdir.join("export")
     monkeypatch.setattr("devpi_server.main.configure_logging", lambda a: None)
-    ret = main([
-        "devpi-server",
-        "--serverdir", empty.strpath,
-        "--export", export.strpath])
+    if use_option:
+        ret = main([
+            "devpi-server",
+            "--serverdir", empty.strpath,
+            "--export", export_dir.strpath])
+    else:
+        ret = export(argv=[
+            "devpi-export",
+            "--serverdir", empty.strpath,
+            export_dir.strpath])
     out, err = capfd.readouterr()
     assert empty.listdir() == []
     assert ret == 1
@@ -84,14 +95,22 @@ def test_export_empty_serverdir(tmpdir, capfd, monkeypatch):
     assert ("The path '%s' contains no devpi-server data" % empty) in err
 
 
-def test_export_import(tmpdir, capfd, monkeypatch):
+@pytest.mark.parametrize("use_option", (False, True))
+def test_export_import(tmpdir, capfd, monkeypatch, use_option):
+    from devpi_server.importexport import import_
+    from devpi_server.init import init
     from devpi_server.main import main
     monkeypatch.setattr("devpi_server.main.configure_logging", lambda a: None)
     clean = tmpdir.join("clean").ensure(dir=True)
-    ret = main([
-        "devpi-server",
-        "--serverdir", clean.strpath,
-        "--init"])
+    if use_option:
+        ret = main(argv=[
+            "devpi-server",
+            "--init",
+            "--serverdir", clean.strpath])
+    else:
+        ret = init(argv=[
+            "devpi-init",
+            "--serverdir", clean.strpath])
     assert ret == 0
     export = tmpdir.join("export")
     ret = main([
@@ -99,28 +118,44 @@ def test_export_import(tmpdir, capfd, monkeypatch):
         "--serverdir", clean.strpath,
         "--export", export.strpath])
     assert ret == 0
-    import_ = tmpdir.join("import")
-    ret = main([
-        "devpi-server",
-        "--serverdir", import_.strpath,
-        "--no-events",
-        "--import", export.strpath])
+    import_dir = tmpdir.join("import")
+    if use_option:
+        ret = main(argv=[
+            "devpi-server",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            "--import", export.strpath])
+    else:
+        ret = import_(argv=[
+            "devpi-import",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            export.strpath])
     assert ret == 0
     out, err = capfd.readouterr()
-    assert os.listdir(clean.strpath) == os.listdir(import_.strpath)
+    assert os.listdir(clean.strpath) == os.listdir(import_dir.strpath)
     assert 'import_all: importing finished' in out
     assert err == ''
 
 
-def test_export_import_no_root_pypi(tmpdir, capfd, monkeypatch):
+@pytest.mark.parametrize("use_option", (False, True))
+def test_export_import_no_root_pypi(tmpdir, capfd, monkeypatch, use_option):
+    from devpi_server.importexport import import_
+    from devpi_server.init import init
     from devpi_server.main import main
     monkeypatch.setattr("devpi_server.main.configure_logging", lambda a: None)
     clean = tmpdir.join("clean").ensure(dir=True)
-    ret = main([
-        "devpi-server",
-        "--serverdir", clean.strpath,
-        "--no-root-pypi",
-        "--init"])
+    if use_option:
+        ret = main(argv=[
+            "devpi-server",
+            "--init",
+            "--serverdir", clean.strpath,
+            "--no-root-pypi"])
+    else:
+        ret = init(argv=[
+            "devpi-init",
+            "--serverdir", clean.strpath,
+            "--no-root-pypi"])
     assert ret == 0
     export = tmpdir.join("export")
     ret = main([
@@ -129,28 +164,43 @@ def test_export_import_no_root_pypi(tmpdir, capfd, monkeypatch):
         "--export", export.strpath])
     assert ret == 0
     # first we test regular import
-    import_ = tmpdir.join("import")
-    ret = main([
-        "devpi-server",
-        "--serverdir", import_.strpath,
-        "--no-events",
-        "--import", export.strpath])
+    import_dir = tmpdir.join("import")
+    if use_option:
+        ret = main(argv=[
+            "devpi-server",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            "--import", export.strpath])
+    else:
+        ret = import_(argv=[
+            "devpi-import",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            export.strpath])
     assert ret == 0
     out, err = capfd.readouterr()
-    assert os.listdir(clean.strpath) == os.listdir(import_.strpath)
+    assert os.listdir(clean.strpath) == os.listdir(import_dir.strpath)
     assert 'import_all: importing finished' in out
     assert err == ''
     # now we add --no-root-pypi
-    import_.remove()
-    ret = main([
-        "devpi-server",
-        "--serverdir", import_.strpath,
-        "--no-events",
-        "--no-root-pypi",
-        "--import", export.strpath])
+    import_dir.remove()
+    if use_option:
+        ret = main(argv=[
+            "devpi-server",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            "--no-root-pypi",
+            "--import", export.strpath])
+    else:
+        ret = import_(argv=[
+            "devpi-import",
+            "--serverdir", import_dir.strpath,
+            "--no-events",
+            "--no-root-pypi",
+            export.strpath])
     assert ret == 0
     out, err = capfd.readouterr()
-    assert os.listdir(clean.strpath) == os.listdir(import_.strpath)
+    assert os.listdir(clean.strpath) == os.listdir(import_dir.strpath)
     assert 'import_all: importing finished' in out
     assert err == ''
 
@@ -162,6 +212,7 @@ def test_import_on_existing_server_data(tmpdir, xom):
     with pytest.raises(Fatal):
         do_import(tmpdir, xom)
 
+
 class TestIndexTree:
     def test_basic(self):
         tree = IndexTree()
@@ -184,39 +235,84 @@ class TestIndexTree:
 
 
 class TestImportExport:
-    @pytest.fixture
-    def makeimpexp(self, makemapp, gentmp):
+    @pytest.fixture(params=[False, True])  # switch between cmds and options
+    def makeimpexp(self, request, makemapp, gentmp, storage_info):
         class ImpExp:
             def __init__(self, options=()):
                 self.exportdir = gentmp()
                 self.testdatadir = py.path.local(
                     pkg_resources.resource_filename(
                         'test_devpi_server', 'importexportdata'))
-                self.mapp1 = makemapp(
-                    options=("--export", self.exportdir) + options)
+                if request.param:
+                    self.mapp1 = makemapp(
+                        options=("--export", self.exportdir) + options)
+                else:
+                    from devpi_server.main import set_state_version
+                    self.mapp1 = makemapp()
+                    set_state_version(self.mapp1.xom.config)
+                    self.options = options
 
             def export(self, initnodeinfo=True):
-                if initnodeinfo:
-                    self.mapp1.xom.config.init_nodeinfo()
-                assert self.mapp1.xom.main() == 0
+                if request.param:
+                    if initnodeinfo:
+                        self.mapp1.xom.config.init_nodeinfo()
+                    assert self.mapp1.xom.main() == 0
+                else:
+                    from devpi_server.importexport import export
+                    argv = [
+                        "devpi-export",
+                        "--serverdir", self.mapp1.xom.config.serverdir]
+                    argv.extend(self.options)
+                    argv.extend(["--storage", storage_info["name"]])
+                    argv.append(self.exportdir)
+                    assert export(argv=argv) == 0
 
-            def import_testdata(self, name, options=(), plugin=None):
+            def import_testdata(self, name, options=()):
                 path = self.testdatadir.join(name).strpath
-                mapp = makemapp(
-                    options=("--import", path) + options)
-                if plugin is not None:
-                    mapp.xom.config.pluginmanager.register(plugin)
-                mapp.xom.config.init_nodeinfo()
-                assert mapp.xom.main() == 0
+                if request.param:
+                    mapp = makemapp(
+                        options=("--import", path) + options)
+                    mapp.xom.config.init_nodeinfo()
+                    assert mapp.xom.main() == 0
+                else:
+                    from devpi_server.importexport import import_
+                    serverdir = gentmp()
+                    argv = [
+                        "devpi-import",
+                        "--no-events",
+                        "--serverdir", serverdir]
+                    argv.extend(options)
+                    argv.extend(["--storage", storage_info["name"]])
+                    argv.append(path)
+                    assert import_(argv=argv) == 0
+                    mapp = makemapp(options=["--serverdir", serverdir])
                 return mapp
 
             def new_import(self, options=(), plugin=None):
-                mapp2 = makemapp(
-                    options=("--import", str(self.exportdir)) + options)
-                if plugin is not None:
-                    mapp2.xom.config.pluginmanager.register(plugin)
-                mapp2.xom.config.init_nodeinfo()
-                assert mapp2.xom.main() == 0
+                if request.param:
+                    mapp2 = makemapp(
+                        options=("--import", str(self.exportdir)) + options)
+                    if plugin is not None:
+                        mapp2.xom.config.pluginmanager.register(plugin)
+                    mapp2.xom.config.init_nodeinfo()
+                    assert mapp2.xom.main() == 0
+                else:
+                    from devpi_server.config import get_pluginmanager
+                    from devpi_server.importexport import import_
+                    serverdir = gentmp()
+                    argv = [
+                        "devpi-import",
+                        "--no-events",
+                        "--serverdir", serverdir]
+                    argv.extend(options)
+                    argv.extend(["--storage", storage_info["name"]])
+                    argv.append(self.exportdir)
+                    pm = get_pluginmanager()
+                    pm.register(plugin)
+                    assert import_(pluginmanager=pm, argv=argv) == 0
+                    mapp2 = makemapp(options=["--serverdir", serverdir])
+                    if plugin is not None:
+                        mapp2.xom.config.pluginmanager.register(plugin)
                 return mapp2
         return ImpExp
 
@@ -261,7 +357,6 @@ class TestImportExport:
         indexlist = mapp2.getindexlist(api.user)
         assert stagename2 in indexlist
         assert indexlist[stagename2]["bases"] == [api.stagename]
-        assert mapp2.xom.config.secret == mapp1.xom.config.secret
 
     def test_indexes_custom_data(self, impexp):
         mapp1 = impexp.mapp1
@@ -306,12 +401,27 @@ class TestImportExport:
             stage = mapp.xom.model.getstage('root/dev')
             assert stage.ixconfig['bases'] == ('root/dev',)
 
+    def test_deleted_base(self, caplog, impexp):
+        mapp = impexp.import_testdata('deletedbase')
+        with mapp.xom.keyfs.transaction(write=False):
+            assert mapp.xom.model.getstage('root/removed') is None
+            stage = mapp.xom.model.getstage('root/dev1')
+            assert stage.ixconfig['bases'] == ('root/removed',)
+            stage = mapp.xom.model.getstage('root/dev2')
+            assert stage.ixconfig['bases'] == ('root/removed',)
+            stage = mapp.xom.model.getstage('root/dev3')
+            assert stage.ixconfig['bases'] == ('root/dev2',)
+            stage = mapp.xom.model.getstage('root/dev4')
+            assert stage.ixconfig['bases'] == ('root/removed', 'root/pypi')
+            stage = mapp.xom.model.getstage('root/dev5')
+            assert stage.ixconfig['bases'] == ('root/removed', 'root/dev2')
+
     def test_bad_username(self, caplog, impexp):
         with pytest.raises(SystemExit):
             impexp.import_testdata('badusername')
         (record,) = caplog.getrecords('contains characters')
         assert 'root~foo.com' in record.message
-        (record,) = caplog.getrecords('You should edit')
+        (record,) = caplog.getrecords('You could also try to edit')
         assert 'dataindex.json' in record.message
 
     def test_bad_indexname(self, caplog, impexp):
@@ -319,9 +429,97 @@ class TestImportExport:
             impexp.import_testdata('badindexname')
         (record,) = caplog.getrecords('contains characters')
         assert 'root/pypi!Jo' in record.message
-        (record,) = caplog.getrecords('You should edit')
+        (record,) = caplog.getrecords('You could also try to edit')
         assert 'dataindex.json' in record.message
 
+    @pytest.mark.parametrize("norootpypi", [False, True])
+    def test_import_no_user(self, caplog, impexp, norootpypi):
+        from devpi_server.main import _pypi_ixconfig_default
+        options = ()
+        if norootpypi:
+            options = ('--no-root-pypi',)
+        mapp = impexp.import_testdata('nouser', options=options)
+        with mapp.xom.keyfs.transaction(write=False):
+            user = mapp.xom.model.get_user("root")
+            assert user is not None
+            stage = mapp.xom.model.getstage("root/pypi")
+            if norootpypi:
+                assert stage is None
+            else:
+                assert stage.ixconfig == _pypi_ixconfig_default
+
+    @pytest.mark.parametrize("norootpypi", [False, True])
+    def test_import_no_root_pypi(self, caplog, impexp, norootpypi):
+        from devpi_server.main import _pypi_ixconfig_default
+        options = ()
+        if norootpypi:
+            options = ('--no-root-pypi',)
+        mapp = impexp.import_testdata('nouser', options=options)
+        with mapp.xom.keyfs.transaction(write=False):
+            user = mapp.xom.model.get_user("root")
+            assert user is not None
+            stage = mapp.xom.model.getstage("root/pypi")
+            if norootpypi:
+                assert stage is None
+            else:
+                assert stage.ixconfig == _pypi_ixconfig_default
+
+    def test_include_mirrordata(self, caplog, makeimpexp, maketestapp, pypistage):
+        impexp = makeimpexp(options=('--include-mirrored-files',))
+        mapp1 = impexp.mapp1
+        testapp = maketestapp(mapp1.xom)
+        api = mapp1.use('root/pypi')
+        pypistage.mock_simple(
+            "package",
+            '<a href="/package-1.0.zip" />\n'
+            '<a href="/package-1.1.zip#sha256=a665a45920422f9d417e4867efdc4fb8a04a1f3fff1fa07e998e86f7f7a27ae3" />\n'
+            + '<a href="/package-2.0.zip#sha256=b3a8e0e1f9ab1bfe3a36f231f676f78bb30a519d2b21e6c530c0eee8ebb4a5d0" data-requires-python="&gt;=3.5" />')
+        pypistage.mock_extfile("/package-1.1.zip", b"123")
+        pypistage.mock_extfile("/package-2.0.zip", b"456")
+        r = testapp.get(api.index + "/+simple/package/")
+        assert r.status_code == 200
+        # fetch some files, so they are included in the dump
+        (_, link1, link2) = sorted(x.attrs['href'] for x in r.html.select('a'))
+        baseurl = URL(r.request.url)
+        r = testapp.get(baseurl.joinpath(link1).url)
+        assert r.body == b"123"
+        r = testapp.get(baseurl.joinpath(link2).url)
+        assert r.body == b"456"
+        impexp.export()
+        mapp2 = impexp.new_import()
+        with mapp2.xom.keyfs.transaction(write=False):
+            stage = mapp2.xom.model.getstage(api.stagename)
+            stage.offline = True
+            projects = stage.list_projects_perstage()
+            assert projects == {'package'}
+            links = sorted(get_mutable_deepcopy(
+                stage.get_simplelinks_perstage("package")))
+            assert links == [
+                ('package-1.1.zip', 'root/pypi/+f/a66/5a45920422f9d/package-1.1.zip', None),
+                ('package-2.0.zip', 'root/pypi/+f/b3a/8e0e1f9ab1bfe/package-2.0.zip', '>=3.5')]
+
+    def test_mirrordata(self, caplog, impexp):
+        mapp = impexp.import_testdata('mirrordata')
+        with mapp.xom.keyfs.transaction(write=False):
+            stage = mapp.xom.model.getstage('root/pypi')
+            stage.offline = True
+            (link,) = stage.get_simplelinks_perstage("dddttt")
+            link = stage.get_link_from_entrypath(link[1])
+            assert link.project == "dddttt"
+            assert link.version == "0.1.dev1"
+            assert link.relpath == 'root/pypi/+f/100/7f0cd10aaa290/dddttt-0.1.dev1.tar.gz'
+            assert link.entry.hash_spec == 'sha256=1007f0cd10aaa290eb84f092e786639bbe930b7f2169f51f755a0f50a6aba489'
+
+    def test_modifiedpypi(self, caplog, impexp):
+        mapp = impexp.import_testdata('modifiedpypi')
+        with mapp.xom.keyfs.transaction(write=False):
+            stage = mapp.xom.model.getstage('root/pypi')
+            # test that we actually get the config from the import and not
+            # the default PyPI settings
+            assert stage.ixconfig['title'] == 'Modified PyPI'
+            assert stage.ixconfig['mirror_url'] == 'https://example.com/simple/'
+            assert stage.ixconfig['mirror_web_url_fmt'] == 'https://example.com/project/{name}/'
+
     def test_normalization(self, caplog, impexp):
         mapp = impexp.import_testdata('normalization')
         with mapp.xom.keyfs.transaction(write=False):
@@ -349,8 +547,26 @@ class TestImportExport:
             link = stage.get_link_from_entrypath(links[1].entrypath)
             assert link.entry.file_get_content() == b"content2"
 
-    def test_upload_releasefile_with_toxresult(self, impexp):
-        from test_devpi_server.example import tox_result_data
+    def test_removed_index_plugin(self, caplog, impexp):
+        # when a plugin for an index type is removed, the name is unknown
+        # here we test that this throws an error
+        with pytest.raises(SystemExit) as e:
+            impexp.import_testdata('removedindexplugin')
+        assert e.value.args == (1,)
+        (record,) = [
+            x for x in caplog.get_records("call")
+            if 'Unknown index type' in x.message]
+        assert "--skip-import-type custom" in record.message
+        # now we test that we can skip the import
+        mapp = impexp.import_testdata(
+            'removedindexplugin', options=('--skip-import-type', 'custom'))
+        with mapp.xom.keyfs.transaction(write=False):
+            stage = mapp.xom.model.getstage('user/dev')
+            assert stage is None
+            userconfig = mapp.xom.model.get_user('user').get()
+            assert userconfig['indexes'] == {}
+
+    def test_upload_releasefile_with_toxresult(self, impexp, tox_result_data):
         mapp1 = impexp.mapp1
         api = mapp1.create_and_use()
         content = b'content'
@@ -383,8 +599,7 @@ class TestImportExport:
             assert history_log[0]['who'] == 'user1'
             assert history_log[0]['dst'] == 'user1/dev'
 
-    def test_import_without_history_log(self, impexp):
-        from test_devpi_server.example import tox_result_data
+    def test_import_without_history_log(self, impexp, tox_result_data):
         DUMP_FILE = {
           "users": {
             "root": {
@@ -619,7 +834,7 @@ class TestImportExport:
                   "username": "user1",
                   "pwsalt": "RMAM7ycp8aqw4vytBOBEKA==",
                   "pwhash": "d9f98f41f8cbdeb6a30a7b6c376d0ccdd76e862ad1fa508b79d4c2098cc9d69a"
-             }
+              }
           }
         }
         with open(impexp.exportdir.join('dataindex.json').strpath, 'w') as fp:
@@ -705,7 +920,7 @@ class TestImportExport:
     @pytest.mark.storage_with_filesystem
     @pytest.mark.skipif(not hasattr(os, 'link'),
                         reason="OS doesn't support hard links")
-    def test_hard_links(self, makeimpexp):
+    def test_export_hard_links(self, makeimpexp):
         impexp = makeimpexp(options=('--hard-links',))
         mapp1 = impexp.mapp1
         api = mapp1.create_and_use()
@@ -714,8 +929,10 @@ class TestImportExport:
         content = zip_dict({"index.html": "<html/>"})
         mapp1.upload_doc("he-llo.zip", content, "he-llo", "")
 
+        # export the data
         impexp.export()
 
+        # check the number of links of the files in the exported data
         assert impexp.exportdir.join(
           'dataindex.json').stat().nlink == 1
         assert impexp.exportdir.join(
@@ -723,8 +940,10 @@ class TestImportExport:
         assert impexp.exportdir.join(
           'user1', 'dev', 'he-llo', '1.0', 'he-llo-1.0.tar.gz').stat().nlink == 2
 
+        # now import the data
         mapp2 = impexp.new_import()
 
+        # and check that the files have the expected content
         with mapp2.xom.keyfs.transaction():
             stage = mapp2.xom.model.getstage(api.stagename)
             verdata = stage.get_versiondata_perstage("he_llo", "1.0")
@@ -738,6 +957,55 @@ class TestImportExport:
             assert py.builtin._totext(
                 archive.read("index.html"), 'utf-8') == "<html/>"
 
+    @pytest.mark.storage_with_filesystem
+    @pytest.mark.skipif(not hasattr(os, 'link'),
+                        reason="OS doesn't support hard links")
+    def test_import_hard_links(self, makeimpexp):
+        impexp = makeimpexp()
+        mapp1 = impexp.mapp1
+        api = mapp1.create_and_use()
+        content = b'content'
+        mapp1.upload_file_pypi("he-llo-1.0.tar.gz", content, "he_llo", "1.0")
+        content = zip_dict({"index.html": "<html/>"})
+        mapp1.upload_doc("he-llo.zip", content, "he-llo", "")
+
+        # export the data
+        impexp.export()
+
+        # check the number of links of the files in the exported data
+        assert impexp.exportdir.join(
+            'dataindex.json').stat().nlink == 1
+        assert impexp.exportdir.join(
+            'user1', 'dev', 'he_llo-1.0.doc.zip').stat().nlink == 1
+        assert impexp.exportdir.join(
+            'user1', 'dev', 'he-llo', '1.0', 'he-llo-1.0.tar.gz').stat().nlink == 1
+
+        # now import the data
+        mapp2 = impexp.new_import(options=('--hard-links',))
+
+        # check that the files have the expected content
+        with mapp2.xom.keyfs.transaction():
+            stage = mapp2.xom.model.getstage(api.stagename)
+            verdata = stage.get_versiondata_perstage("he_llo", "1.0")
+            assert verdata["version"] == "1.0"
+            links = stage.get_releaselinks("he_llo")
+            assert len(links) == 1
+            assert links[0].entry.file_get_content() == b'content'
+            # assert os.stat(links[0].entry.file_os_path()).st_nlink == 2
+            doczip = stage.get_doczip("he_llo", "1.0")
+            archive = Archive(py.io.BytesIO(doczip))
+            assert 'index.html' in archive.namelist()
+            assert py.builtin._totext(
+                archive.read("index.html"), 'utf-8') == "<html/>"
+
+        # and the exported files should now have additional links
+        assert impexp.exportdir.join(
+            'dataindex.json').stat().nlink == 1
+        assert impexp.exportdir.join(
+            'user1', 'dev', 'he_llo-1.0.doc.zip').stat().nlink == 2
+        assert impexp.exportdir.join(
+            'user1', 'dev', 'he-llo', '1.0', 'he-llo-1.0.tar.gz').stat().nlink == 2
+
     def test_uploadtrigger_jenkins_removed_if_not_set(self, impexp):
         mapp1 = impexp.mapp1
         api = mapp1.create_and_use()
@@ -828,11 +1096,6 @@ class TestImportExport:
             volatile=True,
             mirror_url="http://localhost:6543/index/",
             mirror_cache_expiry=600,
-            # XXX backward compatibility with devpi-client <= 2.4.1
-            acl_upload=[],
-            bases=[],
-            pypi_whitelist=[],
-            # XXX end
             projects=[])
 
     @pytest.mark.nomockprojectsremote
@@ -864,9 +1127,4 @@ class TestImportExport:
             type="mirror",
             volatile=True,
             mirror_url="http://localhost:6543/index/",
-            # XXX backward compatibility with devpi-client <= 2.4.1
-            acl_upload=[],
-            bases=[],
-            pypi_whitelist=[],
-            # XXX end
             projects=[])
