@@ -1,5 +1,7 @@
 from __future__ import unicode_literals
 import getpass
+import inspect
+import itertools
 import py
 import pytest
 import json
@@ -8,8 +10,12 @@ from devpi_common.metadata import splitbasename
 from devpi_common.archive import Archive, zip_dict
 from devpi_server.config import hookimpl
 from devpi_server.model import InvalidIndexconfig
-from devpi_server.model import get_indexconfig
+from devpi_server.model import PrivateStage
+from devpi_server.model import ensure_boolean
+from devpi_server.model import ensure_acl_list
+from devpi_server.model import ensure_list
 from devpi_server.model import run_passwd
+from devpi_server.model import StageCustomizer
 from py.io import BytesIO
 
 pytestmark = [pytest.mark.writetransaction]
@@ -157,7 +163,7 @@ class TestStage:
 
     def test_indexconfig_set_throws_on_invalid_base_index(self, stage, user):
         with pytest.raises(InvalidIndexconfig) as excinfo:
-            user.create_stage(index="world", bases=("root/dev/123",),)
+            user.create_stage(index="world3", bases=("root/dev/123",),)
         messages = excinfo.value.messages
         assert len(messages) == 1
         assert "root/dev/123" in messages[0]
@@ -297,20 +303,6 @@ class TestStage:
         assert len(links) == 1
         assert links[0].entrypath.endswith("someproject-1.0.zip")
 
-    def test_get_releaselinks_inheritance_shadow_egg(self, pypistage, stage):
-        stage.modify(bases=("root/pypi",), mirror_whitelist=['py'])
-        pypistage.mock_simple("py",
-        """<a href="http://bb.org/download/py.zip#egg=py-dev" />
-           <a href="http://bb.org/download/master#egg=py-dev2" />
-        """)
-        register_and_store(stage, "py-1.0.tar.gz", b"123")
-        links = stage.get_releaselinks("py")
-        assert len(links) == 3
-        e0, e1, e2 = links
-        assert e0.basename == "py-1.0.tar.gz"
-        assert e1.basename == "py.zip"
-        assert e2.basename == "master"
-
     def test_inheritance_error_are_nop(self, pypistage, stage):
         stage.modify(bases=("root/pypi",), mirror_whitelist=['someproject'])
         pypistage.mock_simple("someproject", status_code = -1)
@@ -503,23 +495,51 @@ class TestStage:
         ('foo,bar', ['foo', 'bar']),
         ('*', ['*'])])
     def test_whitelist_setting(self, pypistage, stage, setting, expected):
-        from devpi_server.model import InvalidIndexconfig
         stage.modify(mirror_whitelist=setting)
         ixconfig = stage.get()
-        assert ixconfig['pypi_whitelist'] == []
+        # BBB old devpi versions had pypi_whitelist, here we check that it's gone
+        assert 'pypi_whitelist' not in ixconfig
         assert ixconfig['mirror_whitelist'] == expected
-        stage.modify(pypi_whitelist=setting)
-        ixconfig = stage.get()
-        assert ixconfig['pypi_whitelist'] == []
-        assert ixconfig['mirror_whitelist'] == expected
-        stage.modify(pypi_whitelist=[], mirror_whitelist=setting)
-        ixconfig = stage.get()
-        assert ixconfig['pypi_whitelist'] == []
-        assert ixconfig['mirror_whitelist'] == expected
-        with pytest.raises(InvalidIndexconfig):
-            stage.modify(pypi_whitelist=setting, mirror_whitelist=[])
-        with pytest.raises(InvalidIndexconfig):
-            stage.modify(pypi_whitelist=setting, mirror_whitelist=setting)
+
+    def test_legacy_pypi_whitelist(self, stage):
+        assert stage.ixconfig['volatile'] is True
+        # now we try to modify the index with the old pypi_whitelist setting
+        # which can still exist in dbs and will be sent by devpi-client
+        stage.modify(pypi_whitelist=[], volatile=False)
+        # if all went well, volatile should be changed
+        assert stage.ixconfig['volatile'] is False
+        # and pypi_whitelist ignored
+        assert 'pypi_whitelist' not in stage.ixconfig
+
+    @pytest.mark.notransaction
+    def test_legacy_pypi_whitelist_removed(self, xom):
+        with xom.keyfs.transaction(write=True):
+            user = xom.model.create_user("hello", password="123")
+            config = udict(index="world", bases=(), type="stage", volatile=True)
+            user.create_stage(**config)
+            with user.key.update() as userconfig:
+                # here we inject the legacy setting
+                userconfig['indexes']['world']['pypi_whitelist'] = []
+            del user
+        with xom.keyfs.transaction(write=True):
+            stage = xom.model.getstage('hello/world')
+            assert stage.ixconfig['volatile'] is True
+            assert 'pypi_whitelist' in stage.ixconfig
+            new_config = dict(stage.ixconfig)
+            new_config['volatile'] = False
+            # now we try to modify the index
+            stage.modify(**new_config)
+            assert stage.ixconfig['volatile'] is False
+            assert 'pypi_whitelist' not in stage.ixconfig
+
+    def test_package_not_in_mirror_whitelist_all(self, monkeypatch, pypistage, stage):
+        stage.modify(mirror_whitelist="*", bases=(pypistage.name,))
+        register_and_store(stage, "some_xyz-1.0.zip", b"123")
+        # provoke error if called
+        monkeypatch.setattr(
+            "devpi_server.extpypi.PyPIStage.list_versions_perstage",
+            lambda self, project: 0 / 0)
+        assert stage.list_versions('some_xyz') == {'1.0'}
 
     def test_store_and_delete_project(self, stage):
         register_and_store(stage, "some_xyz-1.0.zip", b"123")
@@ -566,11 +586,10 @@ class TestStage:
 
     def test_set_versiondata_twice(self, stage, bases, caplog):
         stage.set_versiondata(udict(name="pkg1", version="1.0"))
-        assert not caplog.getrecords("ignored")
         stage.xom.keyfs.commit_transaction_in_thread()
         with stage.xom.keyfs.transaction(write=True):
             stage.set_versiondata(udict(name="pkg1", version="1.0"))
-            assert caplog.getrecords("same metadata")
+        assert caplog.getrecords("nothing to commit")
 
     def test_getdoczip(self, stage, bases, tmpdir):
         stage.set_versiondata(udict(name="pkg1", version="1.0"))
@@ -666,7 +685,7 @@ class TestStage:
         tox_links = list(linkstore.get_links(rel="toxresult"))
         assert len(tox_links) == 1
         tentry = tox_links[0].entry
-        assert tentry.basename == "pkg1-1.0.tar.gz.toxresult0"
+        assert tentry.basename.startswith("pkg1-1.0.tar.gz.toxresult-")
         # check that tentry is in the same dir than entry
         assert tentry.relpath.startswith(entry.relpath)
 
@@ -741,7 +760,7 @@ class TestStage:
 
     def test_releasedata_validation(self, stage):
         with pytest.raises(ValueError):
-             stage.set_versiondata(udict(name="hello_", version="1.0"))
+            stage.set_versiondata(udict(name="hello_", version="1.0"))
 
     def test_set_versiondata_take_existing_name_issue84(self, stage, caplog):
         import logging
@@ -866,6 +885,230 @@ class TestStage:
         assert stage.get_versiondata("hello", "1.0")
         assert stage.get_versiondata("This", "1.0")
 
+    @pytest.mark.notransaction
+    def test_get_last_change_serial_perstage(self, xom):
+        current_serial = xom.keyfs.get_current_serial()
+        model = xom.model
+        with xom.keyfs.transaction(write=True):
+            user = model.create_user("hello", password="123")
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        current_serial = xom.keyfs.get_current_serial()
+        with xom.keyfs.transaction(write=True):
+            stage = user.create_stage(**udict(
+                index="world", bases=(), type="stage", volatile=True))
+            with pytest.raises(KeyError) as e:
+                stage.get_last_change_serial_perstage()
+            assert 'not commited yet' in str(e.value)
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        current_serial = xom.keyfs.get_current_serial()
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == current_serial
+        actions = [
+            ('set_versiondata', udict(name="pkg", version="1.0")),
+            ('store_releasefile', "pkg", "1.0", "pkg-1.0.zip", b""),
+            ('store_doczip', "pkg", "1.0", b""),
+            ('set_versiondata', udict(name="hello", version="1.0")),
+            ('store_releasefile', "hello", "1.0", "hello-1.0.zip", b""),
+            ('store_releasefile', "pkg", "1.0", "pkg-1.0.tar.gz", b"")]
+        for action in actions:
+            with xom.keyfs.transaction(write=True):
+                getattr(stage, action[0])(*action[1:])
+                # inside the transaction there is no change yet
+                assert stage.get_last_change_serial_perstage() == current_serial
+            assert current_serial == xom.keyfs.get_current_serial() - 1
+            current_serial = xom.keyfs.get_current_serial()
+            with xom.keyfs.transaction(write=False):
+                assert stage.get_last_change_serial_perstage() == current_serial
+        # create a toxresult
+        with xom.keyfs.transaction(write=True):
+            (link,) = stage.get_releaselinks_perstage('hello')
+            stage.store_toxresult(link, {})
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        current_serial = xom.keyfs.get_current_serial()
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == current_serial
+        # create another stage and run the same actions
+        serial_before_new_stage = xom.keyfs.get_current_serial()
+        with xom.keyfs.transaction(write=True):
+            stage2 = user.create_stage(**udict(
+                index="world2", bases=(), type="stage", volatile=True))
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        current_serial = xom.keyfs.get_current_serial()
+        for action in actions:
+            with xom.keyfs.transaction(write=True):
+                getattr(stage2, action[0])(*action[1:])
+                # inside the transaction there is no change yet
+                assert stage2.get_last_change_serial_perstage() == current_serial
+            assert current_serial == xom.keyfs.get_current_serial() - 1
+            current_serial = xom.keyfs.get_current_serial()
+            with xom.keyfs.transaction(write=False):
+                assert stage2.get_last_change_serial_perstage() == current_serial
+        # the other stage should not have changed
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == serial_before_new_stage
+        # now we test deletions
+        with xom.keyfs.transaction(write=False):
+            (link,) = stage2.get_releaselinks_perstage('hello')
+            entry = link.entry
+        actions = [
+            ('del_entry', entry, False),
+            ('del_versiondata', 'hello', '1.0', False),
+            ('del_project', 'hello')]
+        for action in actions:
+            with xom.keyfs.transaction(write=True):
+                getattr(stage2, action[0])(*action[1:])
+                # inside the transaction there is no change yet
+                assert stage2.get_last_change_serial_perstage() == current_serial
+            assert current_serial == xom.keyfs.get_current_serial() - 1
+            current_serial = xom.keyfs.get_current_serial()
+            with xom.keyfs.transaction(write=False):
+                assert stage2.get_last_change_serial_perstage() == current_serial
+        # the other stage still should not have changed
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == serial_before_new_stage
+        # modifying the stage also should not affect the other one
+        with xom.keyfs.transaction(write=True):
+            stage2.modify(volatile=False)
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        current_serial = xom.keyfs.get_current_serial()
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == serial_before_new_stage
+        # deleting the stage should not affect the other one either
+        with xom.keyfs.transaction(write=True):
+            stage2.delete()
+            del stage2
+        assert current_serial == xom.keyfs.get_current_serial() - 1
+        with xom.keyfs.transaction(write=False):
+            assert stage.get_last_change_serial_perstage() == serial_before_new_stage
+
+    @pytest.mark.notransaction
+    def test_get_last_project_change_serial_perstage(self, xom):
+        with xom.keyfs.transaction(write=True) as tx:
+            user = xom.model.create_user("hello", password="123")
+            stage = user.create_stage(index="world", type="stage")
+        # get_last_project_change_serial_perstage only works with
+        # commited transactions
+        with xom.keyfs.transaction() as tx:
+            first_serial = tx.at_serial
+            assert stage.list_projects_perstage() == set()
+            assert stage.get_last_project_change_serial_perstage('pkg') == -1
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == first_serial
+            stage.add_project_name('pkg')
+            assert stage.list_projects_perstage() == {'pkg'}
+        with xom.keyfs.transaction() as tx:
+            # the addition of the project name updated the db
+            assert tx.at_serial == (first_serial + 1)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 1)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 1)
+            with stage.key_projects.update() as projects:
+                projects.remove('pkg')
+            assert stage.list_projects_perstage() == set()
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the project name updated the db
+            assert tx.at_serial == (first_serial + 2)
+            # and we can't know when it happened, checking all changes in the
+            # project name set would be too expensive
+            assert stage.get_last_project_change_serial_perstage('pkg') == -1
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 2)
+            stage.set_versiondata(udict(name='pkg', version='1.0'))
+            assert stage.list_projects_perstage() == {'pkg'}
+            assert stage.list_versions_perstage('pkg') == {'1.0'}
+        with xom.keyfs.transaction() as tx:
+            # the addition of the version updated the db
+            assert tx.at_serial == (first_serial + 3)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 3)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 3)
+            stage.set_versiondata(udict(name='other', version='1.1'))
+            assert stage.list_projects_perstage() == {'other', 'pkg'}
+            assert stage.list_versions_perstage('other') == {'1.1'}
+        with xom.keyfs.transaction() as tx:
+            # the addition of the version in another project updated the db
+            assert tx.at_serial == (first_serial + 4)
+            # but the checked project didn't change
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 3)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 4)
+            stage.set_versiondata(udict(name='pkg', version='2.0'))
+            assert stage.list_projects_perstage() == {'other', 'pkg'}
+            assert stage.list_versions_perstage('pkg') == {'1.0', '2.0'}
+        with xom.keyfs.transaction() as tx:
+            # the addition of a second version updated the db
+            assert tx.at_serial == (first_serial + 5)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 5)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 5)
+            stage.del_versiondata('pkg', '2.0')
+            assert stage.list_projects_perstage() == {'other', 'pkg'}
+            assert stage.list_versions_perstage('pkg') == {'1.0'}
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the version updated the db
+            assert tx.at_serial == (first_serial + 6)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 6)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 6)
+            link = stage.store_releasefile('pkg', '1.0', 'pkg-1.0.zip', b'123')
+        with xom.keyfs.transaction() as tx:
+            # the upload of the release updated the db
+            assert tx.at_serial == (first_serial + 7)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 7)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 7)
+            other_link = stage.store_releasefile('other', '1.1', 'other-1.1.zip', b'123')
+        with xom.keyfs.transaction() as tx:
+            # the upload of the other release updated the db
+            assert tx.at_serial == (first_serial + 8)
+            # but the checked project didn't change
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 7)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 8)
+            # delete only the release without removing the version
+            stage.del_entry(link.entry, cleanup=False)
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the release updated the db
+            assert tx.at_serial == (first_serial + 9)
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 9)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 9)
+            # delete only the release without removing the version
+            stage.del_entry(other_link.entry, cleanup=False)
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the other release updated the db
+            assert tx.at_serial == (first_serial + 10)
+            # but the checked project didn't change
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 9)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 10)
+            stage.del_project('other')
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the other project updated the db
+            assert tx.at_serial == (first_serial + 11)
+            # but the checked project didn't change
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 9)
+        with xom.keyfs.transaction(write=True) as tx:
+            # no change in db yet
+            assert tx.at_serial == (first_serial + 11)
+            stage.del_project('pkg')
+        with xom.keyfs.transaction() as tx:
+            # the deletion of the project updated the db
+            assert tx.at_serial == (first_serial + 12)
+            # and this time we can know when it was deleted
+            assert stage.get_last_project_change_serial_perstage('pkg') == (first_serial + 12)
+
 
 class TestLinkStore:
     @pytest.fixture
@@ -955,6 +1198,16 @@ class TestUsers:
         assert 'pwsalt' not in userconfig
         assert userconfig['pwhash'].startswith("$argon2")
 
+    def test_create_with_hash(self, model):
+        user = model.get_user("user")
+        assert not user
+        user = model.create_user(
+            "user", None,
+            pwhash="$argon2i$v=19$m=102400,t=2,p=8$F2JMqVVKSSnFGEPIGQOAsA$RXW/kJM94gq00l9QWDG0OA")
+        assert user
+        assert user.validate("password")
+        assert not user.validate("password2")
+
     def test_create_and_delete(self, model):
         user = model.create_user("user", password="password")
         user.delete()
@@ -1021,49 +1274,116 @@ def test_setdefault_indexes(xom, model):
             assert py.builtin._istext(key)
 
 
-@pytest.mark.parametrize("key", ("acl_upload", "bases", "mirror_whitelist", "pypi_whitelist"))
+@pytest.mark.parametrize("key", ("acl_upload", "acl_toxresult_upload", "mirror_whitelist"))
 @pytest.mark.parametrize("value, result", (
     ("", []), ("x,y", ["x", "y"]), ("x,,y", ["x", "y"])))
-def test_get_indexconfig_lists(key, value, result):
-    class hooks:
-        @hookimpl
-        def devpiserver_indexconfig_defaults(self, index_type):
-            return {}
-    kvdict = get_indexconfig(hooks(), type="stage", **{key: value})
-    if key == "pypi_whitelist":
-        # check behaviour of older devpi-client
-        key = "mirror_whitelist"
+def test_get_indexconfig_lists(xom, key, value, result):
+    stage = PrivateStage(
+        xom, "user", "index", {"type": "stage"}, StageCustomizer)
+    kvdict = stage.get_indexconfig_from_kwargs(**{key: value})
     assert kvdict[key] == result
 
 
+def test_ensure_boolean():
+    def upper_lower_case_permutations(value):
+        return tuple(
+            "".join(x)
+            for x in itertools.product(*zip(value.lower(), value.upper())))
+    assert ensure_boolean(True) is True
+    for s in upper_lower_case_permutations("yes"):
+        assert ensure_boolean(s) is True
+    for s in upper_lower_case_permutations("true"):
+        assert ensure_boolean(s) is True
+    assert ensure_boolean(False) is False
+    for s in upper_lower_case_permutations("no"):
+        assert ensure_boolean(s) is False
+    for s in upper_lower_case_permutations("false"):
+        assert ensure_boolean(s) is False
+    with pytest.raises(InvalidIndexconfig):
+        ensure_boolean(None)
+        ensure_boolean("")
+        ensure_boolean("foo")
+        ensure_boolean([])
+        ensure_boolean(["foo"])
+
+
+def test_ensure_list():
+    assert isinstance(ensure_list([]), list)
+    assert isinstance(ensure_list(set()), list)
+    assert isinstance(ensure_list(tuple()), list)
+    assert ensure_list("foo") == ["foo"]
+    assert ensure_list("foo,bar") == ["foo", "bar"]
+    assert ensure_list(" foo , bar ") == ["foo", "bar"]
+    with pytest.raises(InvalidIndexconfig):
+        assert isinstance(ensure_list(None), list)
+        assert isinstance(ensure_list(dict()), list)
+
+
+def test_ensure_acl_list():
+    assert isinstance(ensure_acl_list([]), list)
+    assert isinstance(ensure_acl_list(set()), list)
+    assert isinstance(ensure_acl_list(tuple()), list)
+    assert ensure_acl_list("foo") == ["foo"]
+    assert ensure_acl_list("foo,bar") == ["foo", "bar"]
+    assert ensure_acl_list(" foo , bar ") == ["foo", "bar"]
+    assert ensure_acl_list(" foo , bar ") == ["foo", "bar"]
+    assert ensure_acl_list(" :anonymous: , FOO ,:AuTheNTicated: ") == [
+        ":ANONYMOUS:", "FOO", ":AUTHENTICATED:"]
+    with pytest.raises(InvalidIndexconfig):
+        assert isinstance(ensure_acl_list(None), list)
+        assert isinstance(ensure_acl_list(dict()), list)
+
+
 @pytest.mark.parametrize(["input", "expected"], [
     ({},
      dict(type="stage")),
 
     ({"volatile": "foo"},
+     InvalidIndexconfig),
+
+    ({"volatile": True},
      dict(type="stage", volatile=True)),
 
+    ({"volatile": "true"},
+     dict(type="stage", volatile=True)),
+
+    ({"volatile": "Yes"},
+     dict(type="stage", volatile=True)),
+
+    ({"volatile": False},
+     dict(type="stage", volatile=False)),
+
     ({"volatile": "False"},
      dict(type="stage", volatile=False)),
 
+    ({"volatile": "no"},
+     dict(type="stage", volatile=False)),
+
     ({"volatile": "False", "bases": "root/pypi"},
-     dict(type="stage", volatile=False, bases=["root/pypi"])),
+     dict(type="stage", volatile=False, bases=("root/pypi",))),
 
     ({"volatile": "False", "bases": ["root/pypi"]},
-     dict(type="stage", volatile=False, bases=["root/pypi"])),
+     dict(type="stage", volatile=False, bases=("root/pypi",))),
 
     ({"volatile": "False", "bases": ["root/pypi"], "acl_upload": ["hello"]},
-     dict(type="stage", volatile=False, bases=["root/pypi"],
+     dict(type="stage", volatile=False, bases=("root/pypi",),
           acl_upload=["hello"])),
 
-     ({"volatile": "False", "bases": ["root/pypi"], "acl_toxresult_upload": ["hello"]},
-     dict(type="stage", volatile=False, bases=["root/pypi"],
+    ({"volatile": "False", "bases": ["root/pypi"], "acl_toxresult_upload": ["hello"]},
+     dict(type="stage", volatile=False, bases=("root/pypi",),
           acl_toxresult_upload=["hello"])),
 ])
 def test_get_indexconfig_values(xom, input, expected):
-    class hooks:
-        @hookimpl
-        def devpiserver_indexconfig_defaults(self, index_type):
-            return {}
-    result = get_indexconfig(hooks(), type="stage", **input)
-    assert result == expected
+    stage = PrivateStage(
+        xom, "user", "index", {"type": "stage"}, StageCustomizer)
+    if inspect.isclass(expected) and issubclass(expected, Exception):
+        with pytest.raises(expected):
+            stage.get_indexconfig_from_kwargs(**input)
+    else:
+        expected.setdefault("acl_upload", ["user"])
+        expected.setdefault("acl_toxresult_upload", [":ANONYMOUS:"])
+        expected.setdefault("bases", ())
+        expected.setdefault("mirror_whitelist", [])
+        expected.setdefault("volatile", True)
+        result = stage.get_indexconfig_from_kwargs(**input)
+        assert result == expected
