@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 import hashlib
 import json
+import os
 import pytest
-import py
 from devpi_server.log import thread_pop_log
-from devpi_server.fileutil import load
+from devpi_server.fileutil import loads
 from devpi_server.log import threadlog, thread_push_log
 from devpi_server.replica import H_EXPECTED_MASTER_ID, H_MASTER_UUID
 from devpi_server.replica import H_REPLICA_UUID, H_REPLICA_OUTSIDE_URL
@@ -12,14 +12,11 @@ from devpi_server.replica import FileReplicationError
 from devpi_server.replica import ImportFileReplica
 from devpi_server.replica import MasterChangelogRequest
 from devpi_server.replica import ReplicaThread
-from devpi_server.replica import tween_replica_proxy
+from devpi_server.replica import proxy_view_to_master
+from devpi_server.views import iter_remote_file_replica
 from pyramid.httpexceptions import HTTPNotFound
-from pyramid.response import Response
 
 
-def loads(bytestring):
-    return load(py.io.BytesIO(bytestring))
-
 pytestmark = [pytest.mark.notransaction]
 
 
@@ -42,17 +39,20 @@ class TestChangelog:
     replica_uuid = "111"
     replica_url = "http://qwe"
 
-    @pytest.fixture
-    def reqchangelog(self, testapp):
+    @pytest.fixture(params=[False, True])
+    def reqchangelog(self, request, testapp):
         def reqchangelog(serial):
             req_headers = {H_REPLICA_UUID: self.replica_uuid,
                            H_REPLICA_OUTSIDE_URL: self.replica_url}
-            return testapp.get("/+changelog/%s" % serial, expect_errors=False,
-                                headers=req_headers)
+            url = "/+changelog/%s" % serial
+            use_multi_endpoint = request.param
+            if use_multi_endpoint:
+                url = url + "-"
+            return testapp.get(url, expect_errors=False, headers=req_headers)
         return reqchangelog
 
     def get_latest_serial(self, testapp):
-        r = testapp.get("/+changelog/nop", expect_errors=False)
+        r = testapp.get("/+api", expect_errors=False)
         return int(r.headers["X-DEVPI-SERIAL"])
 
     def test_get_latest_serial(self, testapp, mapp):
@@ -78,16 +78,16 @@ class TestChangelog:
         assert r.status_code == 202
         assert int(r.headers["X-DEVPI-SERIAL"]) == latest_serial
 
-    def test_wait_entry_succeeds(self, blank_request, xom, mapp):
+    def test_wait_serial_succeeds(self, blank_request, xom, mapp):
         mapp.create_user("this", password="p")
         req = blank_request()
         req.registry = {"xom": xom}
         mcr = MasterChangelogRequest(req)
         with xom.keyfs.transaction():
             with pytest.raises(HTTPNotFound):
-                mcr._wait_for_entry(xom.keyfs.get_current_serial() + 10)
-            entry = mcr._wait_for_entry(xom.keyfs.get_current_serial())
-        assert entry
+                mcr._wait_for_serial(xom.keyfs.get_current_serial() + 10)
+            serial = mcr._wait_for_serial(xom.keyfs.get_current_serial())
+        assert serial == 1
 
     def test_master_id_mismatch(self, testapp):
         testapp.xget(400, "/+changelog/0", headers={H_EXPECTED_MASTER_ID:str("123")})
@@ -97,6 +97,50 @@ class TestChangelog:
         testapp.xget(400, "/+changelog/0")
 
 
+class TestMultiChangelog:
+    replica_uuid = "111"
+    replica_url = "http://qwe"
+
+    @pytest.fixture
+    def reqchangelogs(self, request, testapp):
+        def reqchangelogs(serial):
+            req_headers = {H_REPLICA_UUID: self.replica_uuid,
+                           H_REPLICA_OUTSIDE_URL: self.replica_url}
+            url = "/+changelog/%s-" % serial
+            return testapp.get(url, expect_errors=False, headers=req_headers)
+        return reqchangelogs
+
+    def get_latest_serial(self, testapp):
+        r = testapp.get("/+api", expect_errors=False)
+        return int(r.headers["X-DEVPI-SERIAL"])
+
+    def test_multiple_changes(self, mapp, noiter, reqchangelogs, testapp):
+        mapp.create_user("this", password="p")
+        mapp.create_user("that", password="p")
+        latest_serial = self.get_latest_serial(testapp)
+        assert latest_serial > 1
+        r = reqchangelogs(0)
+        body = b''.join(r.app_iter)
+        data = loads(body)
+        assert isinstance(data, list)
+        assert len(data) == (latest_serial + 1)
+        assert "this/.config" in str(data[-2])
+        assert "that/.config" in str(data[-1])
+
+    def test_size_limit(self, mapp, monkeypatch, noiter, reqchangelogs, testapp):
+        monkeypatch.setattr(MasterChangelogRequest, "MAX_REPLICA_CHANGES_SIZE", 1024)
+        mapp.create_and_login_user("this", password="p")
+        for i in range(10):
+            mapp.create_index("this/dev%s" % i)
+        latest_serial = self.get_latest_serial(testapp)
+        assert latest_serial > 1
+        r = reqchangelogs(0)
+        body = b''.join(r.app_iter)
+        data = loads(body)
+        assert isinstance(data, list)
+        assert len(data) < latest_serial
+
+
 def get_raw_changelog_entry(xom, serial):
     with xom.keyfs._storage.get_connection() as conn:
         return conn.get_raw_changelog_entry(serial)
@@ -272,16 +316,7 @@ def test_clean_response_headers(mock):
     assert 'EGG' in headers
 
 
-class TestTweenReplica:
-    def test_nowrite(self, xom, blank_request):
-        l = []
-        def wrapped_handler(request):
-            l.append(xom.keyfs.get_current_serial())
-            return Response("")
-        handler = tween_replica_proxy(wrapped_handler, {"xom": xom})
-        handler(blank_request())
-        assert l == [xom.keyfs.get_current_serial()]
-
+class TestProxyViewToMaster:
     def test_write_proxies(self, makexom, blank_request, reqmock, monkeypatch):
         xom = makexom(["--master", "http://localhost"])
         reqmock.mock("http://localhost/blankpath",
@@ -289,8 +324,9 @@ class TestTweenReplica:
         l = []
         monkeypatch.setattr(xom.keyfs, "wait_tx_serial",
                             lambda x: l.append(x))
-        handler = tween_replica_proxy(None, {"xom": xom})
-        response = handler(blank_request(method="PUT"))
+        request = blank_request(method="PUT")
+        request.registry = dict(xom=xom)
+        response = proxy_view_to_master(None, request)
         assert response.headers.get("X-DEVPI-SERIAL") == "10"
         assert l == [10]
 
@@ -301,8 +337,9 @@ class TestTweenReplica:
         l = []
         monkeypatch.setattr(xom.keyfs, "wait_tx_serial",
                             lambda x: l.append(x))
-        handler = tween_replica_proxy(None, {"xom": xom})
-        response = handler(blank_request(method="PUT"))
+        request = blank_request(method="PUT")
+        request.registry = dict(xom=xom)
+        response = proxy_view_to_master(None, request)
         assert response.status == "200 GOOD"
 
     def test_write_proxies_redirect(self, makexom, blank_request, reqmock, monkeypatch):
@@ -310,32 +347,33 @@ class TestTweenReplica:
                        "--outside-url=http://my.domain"])
         reqmock.mock("http://localhost/blankpath",
                      code=302, headers={"X-DEVPI-SERIAL": "10",
-                                        "location": "http://localhost/hello"}
-        )
+                                        "location": "http://localhost/hello"})
         l = []
         monkeypatch.setattr(xom.keyfs, "wait_tx_serial",
                             lambda x: l.append(x))
-        handler = tween_replica_proxy(None, {"xom": xom})
         # normally the app is wrapped by OutsideURLMiddleware, since this is
         # not the case here, we have to set the host explicitly
-        response = handler(
-            blank_request(method="PUT", headers=dict(host='my.domain')))
+        request = blank_request(method="PUT", headers=dict(host='my.domain'))
+        request.registry = dict(xom=xom)
+        response = proxy_view_to_master(None, request)
         assert response.headers.get("X-DEVPI-SERIAL") == "10"
         assert response.headers.get("location") == "http://my.domain/hello"
         assert l == [10]
 
     def test_hop_headers(self, makexom, blank_request, reqmock, monkeypatch):
         xom = makexom(["--master", "http://localhost"])
-        reqmock.mock("http://localhost/blankpath",
-                     code=200, headers={
-                        "Connection": "Keep-Alive, Foo",
-                        "Foo": "abc",
-                        "Keep-Alive": "timeout=30",
-                        "X-DEVPI-SERIAL": "0"})
+        reqmock.mock(
+            "http://localhost/blankpath",
+            code=200, headers={
+                "Connection": "Keep-Alive, Foo",
+                "Foo": "abc",
+                "Keep-Alive": "timeout=30",
+                "X-DEVPI-SERIAL": "0"})
         monkeypatch.setattr(xom.keyfs, "wait_tx_serial",
                             lambda x: x)
-        handler = tween_replica_proxy(None, {"xom": xom})
-        response = handler(blank_request(method="PUT"))
+        request = blank_request(method="PUT")
+        request.registry = dict(xom=xom)
+        response = proxy_view_to_master(None, request)
         assert 'connection' not in response.headers
         assert 'foo' not in response.headers
         assert 'keep-alive' not in response.headers
@@ -363,17 +401,134 @@ def replay(xom, replica_xom, events=True):
         thread_pop_log("NOTI")
 
 
-class TestFileReplication:
-    @pytest.fixture
-    def replica_xom(self, makexom):
+@pytest.fixture
+def make_replica_xom(makexom):
+    def make_replica_xom(options=()):
         from devpi_server.replica import ReplicationErrors
-        replica_xom = makexom(["--master", "http://localhost"])
+        replica_xom = makexom(["--master", "http://localhost"] + list(options))
         keyfs = replica_xom.keyfs
         replica_xom.errors = ReplicationErrors(replica_xom.config.serverdir)
         for key in (keyfs.STAGEFILE, keyfs.PYPIFILE_NOMD5):
             keyfs.subscribe_on_import(
                 key, ImportFileReplica(replica_xom, replica_xom.errors))
         return replica_xom
+    return make_replica_xom
+
+
+class TestUseExistingFiles:
+    def test_use_existing_files(self, caplog, make_replica_xom, mapp, monkeypatch, tmpdir, xom):
+        # this will be the folder to find existing files in the replica
+        existing_base = tmpdir.join('existing').ensure_dir()
+        # prepare data on master
+        mapp.create_and_use()
+        content1 = mapp.makepkg("hello-1.0.zip", b"content1", "hello", "1.0")
+        mapp.upload_file_pypi("hello-1.0.zip", content1, "hello", "1.0")
+        # get the path of the release
+        (path,) = mapp.get_release_paths('hello')
+        # create the file
+        existing_path = existing_base.join(path)
+        existing_path.dirpath().ensure_dir()
+        existing_path.write_binary(content1)
+        # create the replica with the path to existing files
+        replica_xom = make_replica_xom(options=[
+            '--replica-file-search-path', existing_base.strpath])
+        # now sync the replica, if the file is not found there will be an error
+        # because httpget is mocked
+        replay(xom, replica_xom)
+        assert len(caplog.getrecords('checking existing file')) == 1
+
+    @pytest.mark.storage_with_filesystem
+    @pytest.mark.skipif(not hasattr(os, 'link'),
+                        reason="OS doesn't support hard links")
+    def test_hardlink(self, caplog, make_replica_xom, mapp, monkeypatch, tmpdir, xom):
+        # this will be the folder to find existing files in the replica
+        existing_base = tmpdir.join('existing').ensure_dir()
+        # prepare data on master
+        mapp.create_and_use()
+        content1 = mapp.makepkg("hello-1.0.zip", b"content1", "hello", "1.0")
+        mapp.upload_file_pypi("hello-1.0.zip", content1, "hello", "1.0")
+        # get the path of the release
+        (path,) = mapp.get_release_paths('hello')
+        # create the file
+        existing_path = existing_base.join(path)
+        existing_path.dirpath().ensure_dir()
+        existing_path.write_binary(content1)
+        assert existing_path.stat().nlink == 1
+        # create the replica with the path to existing files and using hard links
+        replica_xom = make_replica_xom(options=[
+            '--replica-file-search-path', existing_base.strpath,
+            '--hard-links'])
+        # now sync the replica, if the file is not found there will be an error
+        # because httpget is mocked
+        replay(xom, replica_xom)
+        assert len(caplog.getrecords('checking existing file')) == 1
+        # check the number of links of the file
+        assert existing_path.stat().nlink == 2
+
+    def test_use_existing_files_bad_data(self, caplog, httpget, make_replica_xom, mapp, monkeypatch, tmpdir, xom):
+        # this will be the folder to find existing files in the replica
+        existing_base = tmpdir.join('existing').ensure_dir()
+        # prepare data on master
+        mapp.create_and_use()
+        content1 = mapp.makepkg("hello-1.0.zip", b"content1", "hello", "1.0")
+        mapp.upload_file_pypi("hello-1.0.zip", content1, "hello", "1.0")
+        # get the path of the release
+        (path,) = mapp.get_release_paths('hello')
+        httpget.mockresponse("http://localhost" + path, content=content1)
+        # create the file
+        existing_path = existing_base.join(path)
+        existing_path.dirpath().ensure_dir()
+        existing_path.write_binary(b"bad_data")
+        # create the replica with the path to existing files
+        replica_xom = make_replica_xom(options=[
+            '--replica-file-search-path', existing_base.strpath])
+        # now sync the replica, if the file is not found there will be an error
+        # because httpget is mocked
+        replay(xom, replica_xom)
+        assert len(caplog.getrecords('checking existing file')) == 1
+        assert len(caplog.getrecords('sha256 mismatch, got ec7d057f450dc963f15978af98b9cdda64aca6751c677e45d4a358fe103dc05b')) == 1
+        with replica_xom.keyfs.transaction(write=False):
+            entry = replica_xom.filestore.get_file_entry(path[1:])
+            assert entry.file_get_content() == content1
+
+    @pytest.mark.storage_with_filesystem
+    @pytest.mark.skipif(not hasattr(os, 'link'),
+                        reason="OS doesn't support hard links")
+    def test_hardlink_bad_data(self, caplog, httpget, make_replica_xom, mapp, monkeypatch, tmpdir, xom):
+        # this will be the folder to find existing files in the replica
+        existing_base = tmpdir.join('existing').ensure_dir()
+        # prepare data on master
+        mapp.create_and_use()
+        content1 = mapp.makepkg("hello-1.0.zip", b"content1", "hello", "1.0")
+        mapp.upload_file_pypi("hello-1.0.zip", content1, "hello", "1.0")
+        # get the path of the release
+        (path,) = mapp.get_release_paths('hello')
+        httpget.mockresponse("http://localhost" + path, content=content1)
+        # create the file
+        existing_path = existing_base.join(path)
+        existing_path.dirpath().ensure_dir()
+        existing_path.write_binary(b"bad_data")
+        assert existing_path.stat().nlink == 1
+        # create the replica with the path to existing files and using hard links
+        replica_xom = make_replica_xom(options=[
+            '--replica-file-search-path', existing_base.strpath,
+            '--hard-links'])
+        # now sync the replica, if the file is not found there will be an error
+        # because httpget is mocked
+        replay(xom, replica_xom)
+        assert len(caplog.getrecords('checking existing file')) == 1
+        assert len(caplog.getrecords('sha256 mismatch, got ec7d057f450dc963f15978af98b9cdda64aca6751c677e45d4a358fe103dc05b')) == 1
+        # check the number of links of the file
+        assert existing_path.stat().nlink == 1
+        with replica_xom.keyfs.transaction(write=False):
+            entry = replica_xom.filestore.get_file_entry(path[1:])
+            assert entry.file_get_content() == content1
+
+
+class TestFileReplication:
+    @pytest.fixture
+    def replica_xom(self, make_replica_xom):
+        return make_replica_xom()
 
     def test_no_set_default_indexes(self, replica_xom):
         assert replica_xom.keyfs.get_current_serial() == -1
@@ -417,6 +572,7 @@ class TestFileReplication:
 
         with xom.keyfs.transaction(write=True):
             entry.file_set_content(content1)
+            assert entry.file_exists()
 
         # first we try to return something wrong
         master_url = replica_xom.config.master_url
@@ -430,11 +586,12 @@ class TestFileReplication:
         assert persisted_errors == replica_xom.errors.errors
         with replica_xom.keyfs.transaction():
             assert not r_entry.file_exists()
-            assert not replica_xom.filestore.storedir.join(r_entry.relpath).exists()
+            assert not replica_xom.config.serverdir.join(r_entry._storepath).exists()
 
         # then we try to return the correct thing
         with xom.keyfs.transaction(write=True):
-            entry.file_set_content(content1)
+            # trigger a change
+            entry.last_modified = 'Fri, 09 Aug 2019 13:15:02 GMT'
         xom.httpget.mockresponse(master_file_path, code=200, content=content1)
         replay(xom, replica_xom)
         assert replica_xom.errors.errors == {}
@@ -472,7 +629,7 @@ class TestFileReplication:
         with xom.keyfs.transaction(write=True):
             entry.file_delete()
             entry.delete()
-        assert not xom.filestore.storedir.join(entry.relpath).exists()
+        assert not xom.config.serverdir.join(entry._storepath).exists()
 
         # and simulate what the master will respond
         xom.httpget.mockresponse(master_file_path, status_code=410)
@@ -522,6 +679,7 @@ class TestFileReplication:
 
     def test_cache_remote_file_fails(self, xom, replica_xom, gen,
                                      pypistage, monkeypatch, reqmock):
+        from devpi_server.filestore import BadGateway
         l = []
         monkeypatch.setattr(xom.keyfs, "wait_tx_serial",
                             lambda x: l.append(x))
@@ -534,8 +692,8 @@ class TestFileReplication:
             entry = replica_xom.filestore.get_file_entry(entry.relpath)
             url = replica_xom.config.master_url.joinpath(entry.relpath).url
             pypistage.httpget.mockresponse(url, status_code=500)
-            with pytest.raises(entry.BadGateway) as e:
-                for part in entry.iter_remote_file_replica():
+            with pytest.raises(BadGateway) as e:
+                for part in iter_remote_file_replica(replica_xom, entry):
                     pass
             e.match('received 500 from master')
             e.match('pypi.org/package/some/pytest-1.8.zip: received 404')
@@ -563,7 +721,7 @@ class TestFileReplication:
             pypistage.httpget.mockresponse(url, status_code=500)
             pypistage.httpget.mockresponse(
                 entry.url, headers=headers, content=b'123')
-            result = entry.iter_remote_file_replica()
+            result = iter_remote_file_replica(replica_xom, entry)
             headers = next(result)
             assert headers['content-length'] == '3'
             assert b''.join(result) == b'123'
@@ -610,20 +768,8 @@ class TestFileReplication:
         assert list(replication_errors.errors.keys()) == []
 
 
-def test_should_fetch_remote_file():
-    from devpi_server.views import should_fetch_remote_file
-    from devpi_server.replica import H_REPLICA_FILEREPL
-    class Entry:
-        eggfragment = "some"
-        def file_exists(self):
-            return True
-    assert should_fetch_remote_file(Entry(), {})
-    assert not \
-           should_fetch_remote_file(Entry(), {H_REPLICA_FILEREPL: str("YES")})
-
-
 def test_simplelinks_update_updates_projectname(httpget, monkeypatch,
-    pypistage, replica_pypistage, pypiurls, replica_xom, xom):
+        pypistage, replica_pypistage, pypiurls, replica_xom, xom):
 
     pypistage.mock_simple_projects([])
     pypistage.mock_simple("pytest", pkgver="pytest-1.0.zip")
@@ -675,12 +821,13 @@ def test_get_simplelinks_perstage(httpget, monkeypatch, pypistage, replica_pypis
 
     # we patch wait_tx_serial so we can check and replay
     called = []
-    def wait_tx_serial(serial):
+    def wait_tx_serial(serial, timeout=None):
         called.append(True)
         assert xom.keyfs.get_current_serial() == serial
         assert replica_xom.keyfs.get_current_serial() < serial
         replay(xom, replica_xom, events=False)
         assert replica_xom.keyfs.get_current_serial() == serial
+        return True
 
     monkeypatch.setattr(replica_xom.keyfs, 'wait_tx_serial', wait_tx_serial)
     pypiurls.simple = 'http://localhost:3111/root/pypi/+simple/'
@@ -723,3 +870,28 @@ def test_auth_status_master_down(maketestapp, replica_xom, mock):
     assert len(calls) == 1
     assert calls[0][0][1].url == 'http://localhost/+api'
     assert r.json['result']['authstatus'] == ['fail', '', []]
+
+
+def test_master_url_auth(makexom, monkeypatch):
+    from devpi_server.mythread import Shutdown
+    replica_xom = makexom(opts=["--master=http://foo:pass@localhost"])
+    assert replica_xom.config.master_auth == ("foo", "pass")
+    assert replica_xom.config.master_url.url == "http://localhost"
+    replica_xom.create_app()
+
+    results = []
+
+    def get(*args, **kwargs):
+        results.append((args, kwargs))
+        replica_xom.thread_pool.shutdown()
+
+    monkeypatch.setattr(replica_xom.replica_thread.session, "get", get)
+    with pytest.raises(Shutdown):
+        replica_xom.replica_thread.thread_run()
+    assert results[0][1]['auth'] == ("foo", "pass")
+
+
+def test_master_url_auth_with_port(makexom):
+    replica_xom = makexom(opts=["--master=http://foo:pass@localhost:3140"])
+    assert replica_xom.config.master_auth == ("foo", "pass")
+    assert replica_xom.config.master_url.url == "http://localhost:3140"
